/*
 * Copyright (C) 2019 ~ 2020 Uniontech Software Technology Co.,Ltd.
 *
 * Author:     zhanglei <zhanglei@uniontech.com>
 *
 * Maintainer: zhanglei <zhanglei@uniontech.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "agent.h"

#include <QDir>
#include <QDebug>

const QString configPathName = "uos-service-support";

Agent::Agent(QObject *parent)
    : QObject(parent)
{
}

Agent::~Agent()
{
}

void Agent::updateDeviceTreeFile(const QByteArray &data)
{
    QString saveName = "uos-devicetree.json";
    QString savePath = "/usr/share/" + configPathName + "/ini/";
    QDir iniDir;

    if (!iniDir.exists(savePath)) {
        if (!iniDir.mkpath(savePath)) {
            qDebug() << "make filepath fail!";
            return;
        }
    }

    //将程序的执行路径设置到savepath下
    iniDir.setCurrent(savePath);

    //读写json设备文件
    QFile file(saveName);
    if (!file.open(QIODevice::WriteOnly | QIODevice::Text | QFile::Truncate)) {
        qDebug() << "Device Tree File open error!";
        return;
    } else {
        qDebug() << "Device Tree File open success!";
    }

    if (-1 == file.write(data))
        qDebug() << "write data error!";

    file.close();
}

void Agent::updateCustomFile(const QByteArray &data)
{
    QString saveName = "customer.json";
    QString savePath = "/usr/share/" + configPathName + "/ini/";
    QDir iniDir;

    if (!iniDir.exists(savePath)) {
        if (!iniDir.mkpath(savePath)) {
            qDebug() << "make filepath fail!";
            return;
        }
    }

    //将程序的执行路径设置到savepath下
    iniDir.setCurrent(savePath);

    //读写json设备文件
    QFile file(saveName);
    if (!file.open(QIODevice::WriteOnly | QIODevice::Text | QFile::Truncate)) {
        qDebug() << "Custom File open error!";
        return;
    } else {
        qDebug() << "Custom File open success!";
    }

    if (-1 == file.write(data))
        qDebug() << "write data error!";

    file.close();
}

void Agent::downloadQRCode(QString savePath, QString saveName, const QByteArray &data)
{
    QDir iniDir;

    if (!iniDir.exists(savePath)) {
        if (!iniDir.mkpath(savePath)) {
            qDebug() << "make filepath fail!";
            return;
        }
    }
    //保存文件
    QFile file(savePath.append("/").append(saveName));

    if (!file.open(QIODevice::WriteOnly | QIODevice::Text | QFile::Truncate)) {
        qDebug() << "QRCode File open error!";
        return;
    } else {
        qDebug() << "QRCode File open success!";
    }

    if (-1 == file.write(data))
        qDebug() << "write data error!";

    file.close();
}
