/*
 * Copyright (C) 2019 ~ 2020 Uniontech Software Technology Co.,Ltd.
 *
 * Author:     niuningjia <niuningjia@uniontech.com>
 *
 * Maintainer: niuningjia <niuningjia@uniontech.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef CLIENTSOCKET_H
#define CLIENTSOCKET_H

#include <QObject>
#include <QString>
#include <QTimer>
#include <QNetworkConfigurationManager>
#include <QNetworkAccessManager>
#include <QNetworkReply>
#include <QDBusInterface>
#include <QtWebSockets/QWebSocket>

class DNetState : public QObject
{
    Q_OBJECT
public:
    bool getNetState();
    static DNetState *getInstance();

signals:
    void noticeNetWork(bool state);

private slots:
    void noticeNetState(const uint &netState);

private:
    explicit DNetState();
    ~DNetState();

    QDBusInterface *m_netInterface;
    static DNetState *m_Instance;
    bool m_netState;
};

class DSocketClient : public QObject
{
    Q_OBJECT
public:
    explicit DSocketClient(QString serverUrl = Q_NULLPTR, QString serverPort = Q_NULLPTR, QObject *parent = Q_NULLPTR);
    ~DSocketClient();

public slots:
    void createSocket(const QString &custom);
    void closeSocket();
    void sendMessage(const QByteArray &data);
    void sendSession(const QByteArray &data);
    void sendSurveyOption(const QByteArray &data);
    void uploadFile(const QString &filePath, const QString &msgId);
    void sendTicket(const QString &userName, const QByteArray &tickerValue, const QByteArray categoryValue, const QStringList &strFilePath);
    void getSupportWebUrl(QString urlInfo);
    void getDeviceTree(QString urlInfo);
    void getAllCustomerInfo(QString urlInfo);
    void searchCustomerInfo(const QByteArray &byteArray);
    QString getSendUrlInfo();
    void getUserAvatar(QString avatarUrl);
    void createUser(QByteArray data);
    void initServerUrl(const QString &serverUrl, const QString &serverPort);

signals:
    void socketCreated();
    void connectStatus(const quint8 &state);
    void receiveMessage(const QByteArray &data);
    void sessionStart(const QByteArray &data);
    void sessionFault(const int &code, const QString &status);
    void uploadFileRet(const QString &webUrl, const QByteArray &data);
    void sessionClose(const QByteArray &data);
    void receiveMessageRet(const QByteArray &data);
    void initCustomerInfo(const QByteArray &array);
    void initSupportWebUrl(const QString &url);
    void ticketComplete(const float &networkOrder);
    void updateProgress(const QString &proGress);
    void postFaild();
    void customerInfo(const QByteArray &data);
    void unlockConsultation();
    void avatarPixmap(QPixmap pix);

private:
    QString getUserName();
    void renewConnect();
    QString getSign(const QString &time, const QString &userName);
    uint getWebTime();   

private:
    bool m_isConnectStatus;
    bool m_testServer;
    QString m_serverUrl;
    QString m_serverPort;
    QTimer *m_postTimer;
    QWebSocket m_webSocket;
    QTimer *m_timer;
    QString m_webServerUrl;
    QString m_httpUrl;
    QNetworkAccessManager *m_netManager;
    QNetworkRequest m_httpRequest;
    QString m_openApiToken;

private slots:
    void connected();
    void textReceived(const QString &msg);
    void disConnected();
    void requestFinished(QNetworkReply *reply);
    void updateUploadProcess(const int currentProcess, const int totalProcess);
};

#endif // CLIENTSOCKET_H
