/*
 * Copyright (C) 2019 ~ 2020 Uniontech Software Technology Co.,Ltd.
 *
 * Author:     liutianming <liutianming@uniontech.com>
 *
 * Maintainer: liutianming <liutianming@uniontech.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "deepinaccount.h"

#include <QDBusArgument>
#include <QDBusReply>
#include <QJsonObject>
#include <QJsonDocument>

DeepinAccount::DeepinAccount(QObject *parent)
    : QObject (parent)
    , loginStatus(false)
    , m_avatarPath()
    , m_userName("")
    , m_nickName("")
    , m_userId(0)
{
    init();
}

DeepinAccount::~DeepinAccount()
{

}

void DeepinAccount::init()
{
    m_netInterface = new QDBusInterface(DEEPINIDSERVICE, DEEPINIDPATH, DEEPINIDINTERFACE, QDBusConnection::sessionBus());
    QDBusConnection::sessionBus().connect(DEEPINIDSERVICE,
                                         DEEPINIDPATH,
                                         DEEPINIDPROPERTIES,
                                         QLatin1String("PropertiesChanged"),
                                         this,
                                         SLOT(userInfoChanged(QString, QVariantMap, QStringList)));
}

void DeepinAccount::signUp()
{
    m_netInterface->call("Login");
}

void DeepinAccount::exitLogin()
{
    m_netInterface->call("Logout");
}

void DeepinAccount::userInfoChanged(QString str, QVariantMap map, QStringList list)
{
    Q_UNUSED(str);
    Q_UNUSED(list);
    QVariantMap userInfoMap;
    if(!map.contains("UserInfo"))
        return;

    const QDBusArgument &dbusArg = map.value("UserInfo").value<QDBusArgument>();
    dbusArg >> userInfoMap;

    if (userInfoMap.value("IsLoggedIn").toBool()) {
        loginStatus = true;
        m_userName = userInfoMap.value("Username").toString();
        m_avatarPath = userInfoMap.value("ProfileImage").toString();
        m_nickName = userInfoMap.value("Nickname").toString();
        m_userId = userInfoMap.value("UserID").toInt();
        loggedIn(m_userName, m_avatarPath);
        Q_EMIT createUser(spliceUserInfo());
    } else {
        loginStatus = false;
        m_userName.clear();
        m_avatarPath.clear();
        loggedOut();
    }
}

QByteArray DeepinAccount::spliceUserInfo()
{
    QJsonObject userJson;
    userJson.insert("uid", m_userId);
    userJson.insert("username", m_userName);
    userJson.insert("nickname", m_nickName);

    return QJsonDocument(userJson).toJson();
}

void DeepinAccount::initStatus()
{

    QDBusInterface Interface(DEEPINIDSERVICE, DEEPINIDPATH, DEEPINIDPROPERTIES, QDBusConnection::sessionBus());
    QDBusMessage reply = Interface.call("Get", DEEPINIDSERVICE, "UserInfo");
    QVariant variant = reply.arguments().first();
    QDBusArgument argument = variant.value<QDBusVariant>().variant().value<QDBusArgument>();
    QVariantMap userInfo;
    argument >> userInfo;

    if (userInfo.value("IsLoggedIn").toBool()) {
        loginStatus = true;
        m_userName = userInfo.value("Username").toString();
        m_avatarPath = userInfo.value("ProfileImage").toString();
        m_nickName = userInfo.value("Nickname").toString();
        m_userId = userInfo.value("UserID").toInt();
        loggedIn(m_userName, m_avatarPath);
        Q_EMIT createUser(spliceUserInfo());
    } else {
        loginStatus = false;
        m_userName.clear();
        m_avatarPath.clear();
        loggedOut();
    }
}
