/*
 * Copyright (C) 2019 ~ 2020 Uniontech Software Technology Co.,Ltd.
 *
 * Author:     niuningjia <niuningjia@uniontech.com>
 *
 * Maintainer: niuningjia <niuningjia@uniontech.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef DEVICETREE_H
#define DEVICETREE_H

#include <QObject>
#include <QString>
#include <QMap>
#include <QVector>
#include <QStringList>

typedef struct ServiceCustomer {
    QString name;
    QString team;
    QString telephone;
    QString email;
    QString QRCode;
} SeviceCustomer;

class ServiceInfoConfig : public QObject
{
    Q_OBJECT
public:
    static ServiceInfoConfig *getInstance();
    void loadConfigJsonFile();
    void praseJsonArray(const QByteArray &data);
    void writeJsonFile(const QByteArray &data);
    QString getServiceUrl();
    QString getServicePort();

private:
    explicit ServiceInfoConfig();
    ~ServiceInfoConfig();

    QString m_serviceUrl;
    QString m_servicePort;
    static ServiceInfoConfig *m_pInstance;
};

class Devicetree : public QObject
{
    Q_OBJECT
public:
    static Devicetree *getInstance();
    void praseJsonArray(const QByteArray &data);
    void insertCustomerInfo(const QString &key, const ServiceCustomer &data);
    QString saveQRCode(QString urlpath);
    void praseCustomerJsonArray(const QByteArray &data);
    QStringList getSystemNames();
    QStringList getBlockNames(const QString &system);
    QStringList getItemNames(const QString &block);
    ServiceCustomer getCustomService(const QString &index);
    void loadDeviceJsonFile();
    void loadCustomJsonFile();
    void writeCustomerJsonFile();
    void setServerUrl(QString serverUrl, QString serverPort);
    void praseQRcodeDownload(QByteArray data);

signals:
    void updataDeviceTreeCombox();
    void writeDeviceJsonFile(const QByteArray &data);
    void writeCustomJsonFiles(const QByteArray &data);
    void saveFileQRCode(const QString &saveDir, const QString &savename, const QByteArray &data);

private:
    Devicetree();
    ~Devicetree();

    static Devicetree *m_instance;
    bool m_testServer;
    QStringList m_system;
    QString m_serverUrl;
    QString m_serverPort;
    QMap<QString, QStringList> m_systemMap;
    QMap<QString, QStringList> m_blockMap;
    QMap<QString, ServiceCustomer> m_customers;
    QMap<QString, int> m_customerIndex;
};

#endif // DEVICETREE_H
