 /*
 * Copyright (C) 2019 ~ 2020 Uniontech Software Technology Co.,Ltd.
 *
 * Author:     niuningjia <niuningjia@uniontech.com>
 *
 * Maintainer: niuningjia <niuningjia@uniontech.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "mainwindow.h"
#include "servicevandsupportdbus.h"
#include "common/clientsocket.h"
#include "common/devicetree.h"

#include <DApplication>
#include <DMainWindow>
#include <DWidgetUtil>
#include <DGuiApplicationHelper>
#include <DLog>

#include <QApplication>
#include <QTranslator>
#include <QDBusConnection>
#include <QDBusError>
#include <QDebug>

DWIDGET_USE_NAMESPACE

int main(int argc, char *argv[])
{
    DApplication::loadDXcbPlugin();
    DApplication a(argc, argv);
    DGuiApplicationHelper::instance()->setPaletteType(DGuiApplicationHelper::UnknownType);

    a.setAttribute(Qt::AA_UseHighDpiPixmaps);
    a.loadTranslator();
    a.setApplicationName("uos-service-support");
    a.setOrganizationName("deepin");
    a.setApplicationVersion("2.2.0.15");
    a.setProductIcon(QIcon::fromTheme("uss_title"));

    Dtk::Core::DLogManager::registerConsoleAppender();
    Dtk::Core::DLogManager::registerFileAppender();

    //单例模式
    if (!a.setSingleInstance(a.applicationName()))
        exit(-1);

    //获得应用程序名称
    QTranslator translator;
    QString strAppName = QCoreApplication::applicationName();

    translator.load("/usr/share/deepin-" + strAppName + "/translations/" + strAppName + "_" + QLocale::system().name());

    a.installTranslator(&translator);
    a.setApplicationDisplayName(QObject::tr("Support"));
    a.setApplicationDescription(QObject::tr("Support helps users get official services and technical support in various manners."));

    MainWindow *w = Q_NULLPTR;
    //命令启动携带测试服IP和端口切换测试服务器
    if (argc == 3) {
        Devicetree::getInstance()->setServerUrl(argv[1], argv[2]);
        w = new MainWindow(argv[1], argv[2]);

    } else {
        w = new MainWindow("", "");
    }

    Dtk::Widget::moveToCenter(w);
    w->show();

    return a.exec();

}
