/*
 * Copyright (C) 2019 ~ 2020 Uniontech Software Technology Co.,Ltd.
 *
 * Author:     liutianming <liutianming@uniontech.com>
 *
 * Maintainer: liutianming <liutianming@uniontech.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "mainwindow.h"
#include "common/devicetree.h"
#include "common/deepinaccount.h"
#include "common/licensestatus.h"
#include "widgets/centralwidget.h"
#include "widgets/deallogpthread.h"
#include "widgets/usertitlebar.h"
#include "widgets/userlogin.h"

#include <DApplicationHelper>
#include <DTitlebar>

#include <QDBusInterface>
#include <QDebug>

const QSize listviweitemSizelistmode(160, 51);
const int widgetMinimumwidget = 820;
const int widgetMinimumheight = 650;

MainWindow::MainWindow(QString serverUrl, QString serverPort, QWidget *parent)
    : DMainWindow(parent)
    , m_serverUrl(serverUrl)
    , m_serverPort(serverPort)
    , m_itemModel(Q_NULLPTR)
    , m_pthread(new DealLogPthread(this))
    , m_licenseStatus(new LicenseStatus(this))
    , m_deepinAccount(new DeepinAccount(this))
{
    ServiceInfoConfig::getInstance()->loadConfigJsonFile();
    //构造时加载咨询类别文件
    Devicetree::getInstance()->loadDeviceJsonFile();

    setWindowIcon(QIcon::fromTheme("uss_title"));
    DMenu *titleMenu = new DMenu(this->titlebar());
    this->titlebar()->setMenu(titleMenu);

    m_avatarButton = new UserTitleBar(this);
    m_avatarButton->setFocusPolicy(Qt::NoFocus);

    QPalette palette;
    palette.setBrush(m_avatarButton->backgroundRole(), Qt::transparent);
    palette.setColor(m_avatarButton->backgroundRole(), Qt::transparent);
    m_avatarButton->setPalette(palette);
    this->titlebar()->addWidget(m_avatarButton, Qt::AlignRight);

    m_pthread->start();
    initModuleData();
    initUI();
    initWebsocket();
    m_deepinAccount->initStatus();

    connect(m_listView, &DListView::pressed, this, &MainWindow::showClickedPage);
    connect(m_noNetwork, &NoNetworkWidget::changePage, this, [ = ]() {
        m_stackWidget->setCurrentWidget(m_contentUs);

        int nowList = 3;
        QModelIndex indexFromList = m_itemModel->index(nowList, 0);

        m_listView->pressed(indexFromList);
        m_listView->setCurrentIndex(indexFromList);
        beforeIndex = m_listView->currentIndex();
    });
    connect(m_successWidget, &SuccessWidget::jumpSupportPage, this, [ = ]() {
        m_stackWidget->setCurrentWidget(m_selfSupport);

        int nowList = 0;
        QModelIndex indexFromlist = m_itemModel->index(nowList, 0);

        m_listView->pressed(indexFromlist);
        m_listView->setCurrentIndex(indexFromlist);
        beforeIndex = m_listView->currentIndex();
    });
    connect(m_successWidget, &SuccessWidget::jumpConsultationPage, this, [ = ]() {
        if (DNetState::getInstance()->getNetState()) {
            if (m_deepinAccount->loginStatus) {
                m_stackWidget->setCurrentWidget(m_messageConsultation);
            } else {
                m_stackWidget->setCurrentWidget(m_userLogin);
            }
        } else {
            m_stackWidget->setCurrentWidget(m_noNetwork);
            if (m_record != tr("Consults"))
                m_noNetwork->resetValue();
        }

        int nowList = 1;
        QModelIndex indexFromlist = m_itemModel->index(nowList, 0);
        m_listView->pressed(indexFromlist);
        m_listView->setCurrentIndex(indexFromlist);
        beforeIndex = m_listView->currentIndex();
    });
    connect(DNetState::getInstance(), &DNetState::noticeNetWork, this, &MainWindow::networkState);

    int nowList = 0;
    QModelIndex indexFromlist = m_itemModel->index(nowList, 0);

    m_listView->pressed(indexFromlist);
    m_listView->setCurrentIndex(indexFromlist);
    beforeIndex = m_listView->currentIndex();
}

MainWindow::~MainWindow()
{
    Devicetree::getInstance()->writeCustomerJsonFile();
    if (m_webClient != Q_NULLPTR) {
        delete m_webClient;
        m_webClient = Q_NULLPTR;
    }
}

void MainWindow::initModuleData()
{
    m_moduleData = {
        {tr("Self-Service"), QIcon::fromTheme("uss_auto")},
        {tr("Consults"), QIcon::fromTheme("uss_message")},
        {tr("Online Support"), QIcon::fromTheme("uss_onlineHelp")},
        {tr("Contact Us"), QIcon::fromTheme("uss_connectustwo")}
    };
}

void MainWindow::initUI()
{
    QFont pointFont;
    pointFont.setPointSize(10);

    //使用重写鼠标事件后的widget做为中心部件，防止鼠标在非标题栏拖动页面
    CentralWidget *mainWidget = new CentralWidget(this);
    QHBoxLayout *mainLayout = new QHBoxLayout(mainWidget);
    mainLayout->setContentsMargins(0, 0, 0, 0);

    QHBoxLayout *leftLayout = new QHBoxLayout;
    DFrame *leftFrame = new DFrame;
    leftFrame->setLineWidth(0);
    m_listView = new ListView(mainWidget);

    m_listView->setFrameShape(QFrame::Shape::NoFrame);
    m_listView->setEditTriggers(QListView::NoEditTriggers);
    m_listView->setResizeMode(QListView::Adjust);
    m_listView->setAutoScroll(true);
    m_listView->setItemSpacing(3);
    m_listView->setIconSize(QSize(19, 18));
    m_listView->setItemSize(listviweitemSizelistmode);

    DPalette paLette = DApplicationHelper::instance()->palette(m_listView);

    paLette.setBrush(DPalette::ItemBackground, Qt::transparent);
    DApplicationHelper::instance()->setPalette(m_listView, paLette);

    QFont listviewFont;

    listviewFont.setPointSize(50);
    listviewFont.setBold(false);
    listviewFont.setWeight(20);
    m_listView->setFont(listviewFont);

    m_itemModel = new QStandardItemModel(m_listView);

    for (auto it = m_moduleData.cbegin(); it != m_moduleData.cend(); ++it) {
        QStandardItem *item = new QStandardItem;

        item->setFont(pointFont);
        item->setText(it->first);
        item->setIcon(it->second);
        m_itemModel->appendRow(item);
    }

    m_listView->setModel(m_itemModel);
    leftFrame->setLayout(leftLayout);
    leftLayout->addWidget(m_listView);

    m_stackWidget = new QStackedWidget;
    m_messageConsultation = new MessageConsultation;
    m_customerChat = new CustomerChatWidget;
    m_contentUs = new ContactUs;
    m_selfSupport = new SelfSupport;
    m_noNetwork = new NoNetworkWidget;
    m_successWidget = new SuccessWidget;
    m_userLogin = new UserLoginWidget;
    m_deepinAccount = new DeepinAccount;

    m_stackWidget->addWidget(m_messageConsultation);
    m_stackWidget->addWidget(m_customerChat);
    m_stackWidget->addWidget(m_contentUs);
    m_stackWidget->addWidget(m_selfSupport);
    m_stackWidget->addWidget(m_noNetwork);
    m_stackWidget->addWidget(m_successWidget);
    m_stackWidget->addWidget(m_userLogin);
    m_stackWidget->setCurrentWidget(m_selfSupport);

    QHBoxLayout *rightLayout = new QHBoxLayout;
    rightLayout->setContentsMargins(0, 0, 0, 0);
    rightLayout->addWidget(m_stackWidget);

    DFrame *rightFrame = new DFrame;
    rightFrame->setLineWidth(0);
    rightFrame->setContentsMargins(0, 10, 0, 10);
    rightFrame->setLayout(rightLayout);
    leftFrame->setFixedWidth(180);
    rightFrame->setFixedWidth(620);
    mainLayout->addWidget(leftFrame);
    mainLayout->addSpacing(5);
    mainLayout->addWidget(rightFrame);
    mainLayout->addSpacing(8);
    setCentralWidget(mainWidget);

    DTitlebar *titlebarIcon = this->titlebar();
    titlebarIcon->setTitle("");
    titlebarIcon->setIcon(QIcon::fromTheme("uss_title"));
    setFixedSize(QSize(widgetMinimumwidget, widgetMinimumheight));
}

void MainWindow::showClickedPage(const QModelIndex &index)
{
    QString currentIndex("");
    currentIndex = index.data().toString();

    if (currentIndex == tr("Self-Service")) {
        m_stackWidget->setCurrentWidget(m_selfSupport);
    } else if (currentIndex == tr("Consults")) {
        if (DNetState::getInstance()->getNetState()) {
            if (m_deepinAccount->loginStatus) {
                m_stackWidget->setCurrentWidget(m_messageConsultation);
            } else {
                //未登录界面的调取
                m_stackWidget->setCurrentWidget(m_userLogin);
            }
        } else {
            m_stackWidget->setCurrentWidget(m_noNetwork);
            if (m_record != tr("Consults"))
                m_noNetwork->resetValue();
        }
    } else if (currentIndex == tr("Online Support")) {
        m_listView->setCurrentIndex(index);
        if (DNetState::getInstance()->getNetState()) {
            if (LicenseStatus::authorizeStatus) {
                if(m_deepinAccount->loginStatus) {
                    m_stackWidget->setCurrentWidget(m_customerChat);
                } else {
                    //未登录界面的调取
                    m_stackWidget->setCurrentWidget(m_userLogin);
                }
            } else {
                m_listView->setCurrentIndex(beforeIndex);
                popDialog();
                return;
            }
        } else {
            m_stackWidget->setCurrentWidget(m_noNetwork);
            if (m_record != tr("Online Support"))
                m_noNetwork->resetValue();
        }
    } else if (currentIndex == tr("Contact Us")) {
        m_stackWidget->setCurrentWidget(m_contentUs);
    } else {
    }

    beforeIndex = index;
    m_record = currentIndex;
}

void MainWindow::getAllmessage()
{
    QString urlInfo = m_webClient->getSendUrlInfo();
    m_webClient->getDeviceTree(urlInfo);
    m_webClient->getAllCustomerInfo(urlInfo);
    m_webClient->getSupportWebUrl(urlInfo);
}

void MainWindow::getLoginslots()
{
    if (m_stackWidget->currentWidget() == m_userLogin) {
        if (m_listView->currentIndex().data().toString() == tr("Consults"))
            m_stackWidget->setCurrentWidget(m_messageConsultation);
        if (m_listView->currentIndex().data().toString() == tr("Online Support"))
            m_stackWidget->setCurrentWidget(m_customerChat);
    }
}

void MainWindow::getloggedoutslots()
{
    if ((m_stackWidget->currentWidget() == m_messageConsultation) && (m_listView->currentIndex().data().toString() == tr("Consults"))) {
        m_stackWidget->setCurrentWidget(m_userLogin);
    }
    if ((m_stackWidget->currentWidget() == m_customerChat) && (m_listView->currentIndex().data().toString() == tr("Online Support"))) {
        m_stackWidget->setCurrentWidget(m_userLogin);
    }
}

void MainWindow::popDialog()
{
    DDialog remindMessage(this);
    remindMessage.setIcon(QIcon::fromTheme("uss_warning"));
    remindMessage.addButton(tr("Cancel"));
    remindMessage.addButton(tr("Activate"));
    remindMessage.setMessage(tr("Your system is not activated\nTo enjoy this service, please activate first"));
    connect(&remindMessage, &DDialog::buttonClicked, this, [ = ] (int index) {
        if (index == 1)
            m_licenseStatus->wakeLicenseInterface();
    });

    remindMessage.exec();
}

void MainWindow::initWebsocket()
{
    if (m_serverUrl.isEmpty() && m_serverPort.isEmpty()) {
        m_webClient = new DSocketClient();
    } else {
        m_webClient = new DSocketClient(m_serverUrl, m_serverPort);
    }

    connect(this, &MainWindow::initContactQRcode, m_contentUs, &ContactUs::updateComboBox);
    connect(m_webClient, &DSocketClient::customerInfo, Devicetree::getInstance(), &Devicetree::praseQRcodeDownload);
    connect(m_deepinAccount, &DeepinAccount::createUser, m_webClient, &DSocketClient::createUser);
    connect(m_deepinAccount, &DeepinAccount::loggedIn, this, &MainWindow::getLoginslots);
    connect(m_deepinAccount, &DeepinAccount::loggedOut, this, &MainWindow::getloggedoutslots);
    connect(Devicetree::getInstance(), &Devicetree::updataDeviceTreeCombox, m_messageConsultation, &MessageConsultation::updateComboBox);
    connect(m_webClient, &DSocketClient::updateProgress, m_messageConsultation, &MessageConsultation::updateProgress);
    connect(m_webClient, &DSocketClient::ticketComplete, m_messageConsultation, &MessageConsultation::ticketComplete);
    connect(m_webClient, &DSocketClient::postFaild, m_messageConsultation, &MessageConsultation::postFailed);
    connect(m_messageConsultation, &MessageConsultation::signalTicket, m_webClient, &DSocketClient::sendTicket);
    connect(m_webClient, &DSocketClient::receiveMessage, m_customerChat, &CustomerChatWidget::recieveMessage);
    connect(m_webClient, &DSocketClient::sessionStart, m_customerChat, &CustomerChatWidget::sessionStart);
    connect(m_webClient, &DSocketClient::sessionFault, m_customerChat, &CustomerChatWidget::sessionFault);
    connect(m_webClient, &DSocketClient::uploadFileRet, m_customerChat, &CustomerChatWidget::uploadFileSuccess);
    connect(m_webClient, &DSocketClient::receiveMessageRet, m_customerChat, &CustomerChatWidget::recieveMessageRet);
    connect(m_customerChat, &CustomerChatWidget::sendStartSession, m_webClient, &DSocketClient::sendSession);
    connect(m_customerChat, &CustomerChatWidget::createSocket, m_webClient, &DSocketClient::createSocket);
    connect(m_customerChat, &CustomerChatWidget::closeWebSocket, m_webClient, &DSocketClient::closeSocket);
    connect(m_webClient, &DSocketClient::socketCreated, m_customerChat, &CustomerChatWidget::createSocketSlot);
    connect(m_customerChat, &CustomerChatWidget::sendMessageSignals, m_webClient, &DSocketClient::sendMessage);
    connect(m_customerChat, &CustomerChatWidget::uploadFile, m_webClient, &DSocketClient::uploadFile);
    connect(m_customerChat, &CustomerChatWidget::sendSurvey, m_webClient, &DSocketClient::sendSurveyOption);
    connect(Devicetree::getInstance(), &Devicetree::updataDeviceTreeCombox, m_contentUs, &ContactUs::updateComboBox);
    connect(m_contentUs, &ContactUs::searchCustomInfo, m_webClient, &DSocketClient::searchCustomerInfo);
    connect(m_webClient, &DSocketClient::initCustomerInfo, m_contentUs, &ContactUs::updateCustomerInfo);
    connect(m_webClient, &DSocketClient::initSupportWebUrl, m_selfSupport, &SelfSupport::updateWebUrl);
    connect(m_webClient, &DSocketClient::ticketComplete, this, [ = ](qint64 n_tickId) {
        m_messageConsultation->updateProgress("1");
        //提交完成后取消锁定
        m_messageConsultation->setAttribute(Qt::WA_TransparentForMouseEvents, false);
        qDebug() << "交完成后取消锁 tikID" << n_tickId;
        if (n_tickId != 0)
            m_stackWidget->setCurrentWidget(m_successWidget);
    });
    connect(m_messageConsultation, &MessageConsultation::lockConsultation, this, [ = ]() {
        m_messageConsultation->updateProgress("");
        //在工单提交时锁定留言咨询界面，防止提交过程中用户修改操作提交内容
        m_messageConsultation->setAttribute(Qt::WA_TransparentForMouseEvents, true);
    });
    connect(m_webClient, &DSocketClient::unlockConsultation, this, [ = ]() {
        m_messageConsultation->updateProgress("1");
        m_messageConsultation->setAttribute(Qt::WA_TransparentForMouseEvents, false);
    });

    connect(Devicetree::getInstance(), &Devicetree::writeDeviceJsonFile, this, [ = ](const QByteArray & data) {
        QDBusInterface interface("com.deepin.serviceandsupport", "/com/deepin/serviceandsupport/agent", "com.deepin.serviceandsupport.agent", QDBusConnection::systemBus());
        QList<QVariant> argumentList;
        argumentList << data;
        QDBusMessage reply = interface.callWithArgumentList(QDBus::Block, "updateDeviceTreeFile", argumentList);

        if (reply.type() == QDBusMessage::ErrorMessage)
            qDebug() << "D-Bus Error:" << reply.errorMessage();
    });

    connect(m_webClient, &DSocketClient::customerInfo, this, [ = ](QByteArray data) {
        QDBusInterface interface("com.deepin.serviceandsupport", "/com/deepin/serviceandsupport/agent", "com.deepin.serviceandsupport.agent", QDBusConnection::systemBus());
        QList<QVariant> argumentList;
        argumentList << data;
        QDBusMessage reply = interface.callWithArgumentList(QDBus::Block, "updateCustomFile", argumentList);

        if (reply.type() == QDBusMessage::ErrorMessage)
            qDebug() << "D-Bus Error:" << reply.errorMessage();
    });

    //保存二维码图片
    connect(Devicetree::getInstance(), &Devicetree::saveFileQRCode, this, [ = ](const QString & saveDir, const QString & saveName, const QByteArray & data) {
        QDBusInterface interface("com.deepin.serviceandsupport", "/com/deepin/serviceandsupport/agent", "com.deepin.serviceandsupport.agent", QDBusConnection::systemBus());
        QList<QVariant> argumentList;
        argumentList << saveDir << saveName << data;
        QDBusMessage reply = interface.callWithArgumentList(QDBus::NoBlock, "downloadQRCode", argumentList);
        if (reply.type() == QDBusMessage::ErrorMessage) {
            qDebug() << "D-Bus Error:" << reply.errorMessage();
            return;
        }

        Q_EMIT initContactQRcode();
    });
    getAllmessage();
}

void MainWindow::networkState(bool isConnected)
{
    if (isConnected)
        getAllmessage();
}

ListView::ListView(DWidget *parent)
    : DListView(parent)
{

}

ListView::~ListView()
{

}

void ListView::keyPressEvent(QKeyEvent *event)
{
    if ((event->key() == Qt::Key_Down) || (event->key() == Qt::Key_Up))
        return;

    DListView::keyPressEvent(event);
}

void ListView::mousePressEvent(QMouseEvent *event)
{
    if (event->button() == Qt::RightButton)
        return;

    DListView::mousePressEvent(event);
}
