/*
 * Copyright (C) 2019 ~ 2020 Uniontech Software Technology Co.,Ltd.
 *
 * Author:     liutianming <liutianming@uniontech.com>
 *
 * Maintainer: liutianming <liutianming@uniontech.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include "servicevandsupportdbus.h"
#include "widgets/messageconsultationwidget.h"
#include "widgets/customerchatwidget.h"
#include "widgets/contactus.h"
#include "widgets/selfsupport.h"
#include "widgets/nonetworkwidget.h"
#include "widgets/success.h"
#include "common/clientsocket.h"

#include <DMainWindow>
#include <DFrame>
#include <DBackgroundGroup>
#include <DListView>
#include <DIconButton>
#include <DStandardItem>

#include <QStackedWidget>
#include <QStandardItemModel>
#include <QModelIndex>
#include <QMouseEvent>

DWIDGET_USE_NAMESPACE

class CentralWidget;
class DealLogPthread;
class UserTitleBar;
class DeepinAccount;
class LicenseStatus;
class UserLoginWidget;
class ListView;

class MainWindow : public DMainWindow
{
    Q_OBJECT

public:
    explicit MainWindow(QString serverUrl, QString serverPort, QWidget *parent = Q_NULLPTR);
    ~MainWindow();

signals:
    void updateDevicetreeFile(const QByteArray &argumentList);
    void sendQrcodeName(const QString &savePath);
    void initContactQRcode();

private:
    void initUI();
    void initModuleData();
    void initWebsocket();
    void networkState(bool isConnected);

private slots:
    void showClickedPage(const QModelIndex &index);
    void getAllmessage();
    void getLoginslots();
    void getloggedoutslots();
    void popDialog();

private:
    QString m_serverUrl;
    QString m_serverPort;
    QModelIndex beforeIndex;
    ListView *m_listView;
    DSocketClient *m_webClient;
    QStandardItemModel *m_itemModel;
    QStackedWidget *m_stackWidget;
    QList<QPair<QString, QIcon>> m_moduleData;
    MessageConsultation *m_messageConsultation;
    CustomerChatWidget *m_customerChat;
    ContactUs *m_contentUs;
    SelfSupport *m_selfSupport;
    NoNetworkWidget *m_noNetwork;
    SuccessWidget *m_successWidget;
    QString m_record;
    DealLogPthread *m_pthread;
    UserTitleBar *m_avatarButton;
    LicenseStatus *m_licenseStatus;
    UserLoginWidget *m_userLogin;
    DeepinAccount *m_deepinAccount;
};

class ListView : public DListView
{
    Q_OBJECT

public:
    explicit ListView(DWidget *parent = Q_NULLPTR);
    ~ListView();

protected:
    void keyPressEvent(QKeyEvent *event) ;
    void mousePressEvent(QMouseEvent *event);

};

#endif // MAINWINDOW_H
