/*
 * Copyright (C) 2019 ~ 2020 Uniontech Software Technology Co.,Ltd.
 *
 * Author:     zhangminjie <zhangminjie@uniontech.com>
 *
 * Maintainer: zhangminjie <zhangminjie@uniontech.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "contactus.h"
#include "common/clientsocket.h"
#include "common/devicetree.h"

#include <DPalette>
#include <DFontSizeManager>
#include <DCommandLinkButton>

#include <QIcon>
#include <QFont>
#include <QFile>
#include <QDebug>
#include <QPalette>
#include <QListView>
#include <QByteArray>
#include <QJsonValue>
#include <QJsonArray>
#include <QEventLoop>
#include <QJsonObject>
#include <QFileDialog>
#include <QJsonDocument>
#include <QStandardPaths>
#include <QNetworkReply>
#include <QDesktopServices>
#include <QCoreApplication>

const int FONTSIZE = 9;

ContactUs::ContactUs(QWidget *parent)
    : DWidget(parent)
    , m_curSerchKey("")
    , m_netAdress("https://www.chinauos.com")
{
    QFont contactUsFont("SourceHanSansSC-Bold");
    contactUsFont.setPointSize(16);
    contactUsFont.setBold(true);

    DLabel *contactUsLabel = new DLabel;
    contactUsLabel->setFont(contactUsFont);
    contactUsLabel->setText(tr("Contact Us"));

    QFont introduceFont;
    introduceFont.setPointSize(9);
    DPalette color_Introduction;
    color_Introduction.setColor(DPalette::TextTips, QColor("#8AA1B4"));
    DLabel *introduceLabel = new DLabel;
    introduceLabel->setFixedWidth(480);
    introduceLabel->setWordWrap(true);
    introduceLabel->setText(tr("After selecting the category you want to consult below"
                               ", the contact information of the matching support team member will be shown"
                               ". You can call, email, or scan the QR code via your personal WeChat"
                               ", and seek online one-to-one consulting services."));
    introduceLabel->setFont(introduceFont);
    introduceLabel->setPalette(color_Introduction);

    QHBoxLayout *introduceLayout = new QHBoxLayout;
    introduceLayout->addStretch();
    introduceLayout->addWidget(introduceLabel);
    introduceLayout->addStretch();

    //故障树结构
    DFrame *archFrame = createArchFrame();
    //客服名片
    DFrame *cardFrame = createCardFrame();

    QObject::connect(m_consultationTypeOne, &QComboBox::currentTextChanged, this, &ContactUs::onSystemComboBoxIndexChanged);
    QObject::connect(m_consultationTypeTwo, &QComboBox::currentTextChanged, this, &ContactUs::onBlockComboBoxIndexChanged);
    QObject::connect(m_consultationTypeThree, &QComboBox::currentTextChanged, this, &ContactUs::onItemComboBoxIndexChanged);

    QVBoxLayout *mainLayout = new QVBoxLayout;
    mainLayout->setContentsMargins(10, 10, 10, 10);
    mainLayout->addSpacing(50);
    mainLayout->addWidget(contactUsLabel, 0, Qt::AlignCenter);
    mainLayout->addSpacing(5);
    mainLayout->addLayout(introduceLayout);
    mainLayout->addStretch();
    mainLayout->addWidget(archFrame, 0, Qt::AlignCenter);
    mainLayout->addSpacing(16);
    mainLayout->addWidget(cardFrame, 0, Qt::AlignCenter);

    this->setLayout(mainLayout);
    //初始化combobox列表
    initComboBox();
}

ContactUs::~ContactUs()
{
}

DFrame *ContactUs::createCardFrame()
{
    QFont cardFont;
    cardFont.setPointSize(FONTSIZE);
    QPalette colorTitle;
    colorTitle.setColor(QPalette::Text, QColor(82, 106, 127, 255));

    DFrame *cardFrame = new DFrame;
    cardFrame->setFixedSize(600, 304);
    cardFrame->setLineWidth(0);
    cardFrame->setBackgroundRole(DPalette::ItemBackground);

    QVBoxLayout *ussIconLayout = new QVBoxLayout;

    QHBoxLayout *teamLayout = new QHBoxLayout;
    DIconButton *teamIcon = new DIconButton(this);
    teamIcon->setFlat(true);
    teamIcon->setIcon(QIcon::fromTheme("uss_team"));
    teamIcon->setFocusPolicy(Qt::NoFocus);
    teamIcon->setIconSize(QSize(16, 16));
    teamIcon->setFixedSize(QSize(16, 16));

    m_companyLabel = new DLabel;
    m_companyLabel->setText("UnionTech OS Support Team");
    m_companyLabel->setFont(cardFont);
    teamLayout->addWidget(teamIcon);
    teamLayout->addSpacing(10);
    teamLayout->addWidget(m_companyLabel);
    teamLayout->addStretch();

    QHBoxLayout *phoneLayout = new QHBoxLayout;
    DIconButton *phoneIcon = new DIconButton(this);
    phoneIcon->setFlat(true);
    phoneIcon->setIcon(QIcon::fromTheme("uss_connectustwo"));
    phoneIcon->setFocusPolicy(Qt::NoFocus);
    phoneIcon->setIconSize(QSize(16, 16));
    phoneIcon->setFixedSize(QSize(16, 16));

    m_telephoneTextLabel = new DLabel;
    m_telephoneTextLabel->setText("400-8588-488");
    m_telephoneTextLabel->setFont(cardFont);
    phoneLayout->addWidget(phoneIcon);
    phoneLayout->addSpacing(10);
    phoneLayout->addWidget(m_telephoneTextLabel);
    phoneLayout->addStretch();

    QHBoxLayout *mailLayout = new QHBoxLayout;
    DIconButton *mailIcon = new DIconButton(this);
    mailIcon->setFlat(true);
    mailIcon->setIcon(QIcon::fromTheme("uss_mail"));
    mailIcon->setFocusPolicy(Qt::NoFocus);
    mailIcon->setIconSize(QSize(16, 16));
    mailIcon->setFixedSize(QSize(16, 16));

    m_emailAddressLabel = new DLabel;
    m_emailAddressLabel->setText("support@uniontech.com");
    m_emailAddressLabel->setFont(cardFont);
    mailLayout->addWidget(mailIcon);
    mailLayout->addSpacing(10);
    mailLayout->addWidget(m_emailAddressLabel);
    mailLayout->addStretch();

    QHBoxLayout *netAdressLayout = new QHBoxLayout;
    DIconButton *netAdressIcon = new DIconButton(this);
    netAdressIcon->setFlat(true);
    netAdressIcon->setIcon(QIcon::fromTheme("uss_website"));
    netAdressIcon->setFocusPolicy(Qt::NoFocus);
    netAdressIcon->setIconSize(QSize(16, 16));
    netAdressIcon->setFixedSize(QSize(16, 16));

    DCommandLinkButton *netAdressText = new DCommandLinkButton("");
    netAdressText->setFocusPolicy(Qt::NoFocus);
    netAdressText->setText("www.chinauos.com");
    netAdressText->setFont(cardFont);
    netAdressLayout->addWidget(netAdressIcon);
    netAdressLayout->addSpacing(5);
    netAdressLayout->addWidget(netAdressText);
    netAdressLayout->addStretch();
    connect(netAdressText, &DCommandLinkButton::clicked, this, [ = ]() {
        QDesktopServices::openUrl(m_netAdress);
    });

    ussIconLayout->addLayout(teamLayout);
    ussIconLayout->addStretch();
    ussIconLayout->addLayout(phoneLayout);
    ussIconLayout->addStretch();
    ussIconLayout->addLayout(mailLayout);
    ussIconLayout->addStretch();
    ussIconLayout->addLayout(netAdressLayout);

    m_qrcodeLabel = new DLabel;
    QImage image;
    image.load(":/icons/uss_qrcode_135px.jpg");
    //重新调整图像大小以适应label
    QImage qrcImage = image.scaled(QSize(135, 135), Qt::IgnoreAspectRatio);
    m_qrcodeLabel->setPixmap(QPixmap::fromImage(qrcImage));
    m_qrcodeLabel->setFixedSize(m_qrcodeLabel->pixmap()->size());

    QHBoxLayout *cardFrameLayout = new QHBoxLayout;
    cardFrameLayout->setContentsMargins(0, 83, 0, 82);
    cardFrameLayout->addSpacing(54);
    cardFrameLayout->addLayout(ussIconLayout);
    cardFrameLayout->addStretch();
    cardFrameLayout->addWidget(m_qrcodeLabel);
    cardFrameLayout->addSpacing(50);

    cardFrame->setLayout(cardFrameLayout);

    return cardFrame;
}

DFrame *ContactUs::createArchFrame()
{
    QFont archFont;
    archFont.setPointSize(FONTSIZE);

    DFrame *archFrame = new DFrame(this);
    archFrame->setFixedWidth(600);
    archFrame->setLineWidth(0);
    archFrame->setBackgroundRole(DPalette::ItemBackground);

    QHBoxLayout *comboBoxLayout = new QHBoxLayout;
    comboBoxLayout->setContentsMargins(0, 10, 0, 10);

    DLabel *typeLabel = new DLabel;
    typeLabel->setFont(archFont);
    typeLabel->setWordWrap(true);
    typeLabel->setText(tr("Category"));
    comboBoxLayout->addSpacing(16);
    comboBoxLayout->addWidget(typeLabel);

    //初始化下拉按钮
    m_consultationTypeOne = new DComboBox;
    m_consultationTypeOne->setFixedSize(160, 36);
    m_consultationTypeOne->setFont(archFont);
    m_consultationTypeOne->setFocusPolicy(Qt::NoFocus);
    comboBoxLayout->addStretch();
    comboBoxLayout->addWidget(m_consultationTypeOne);

    m_consultationTypeTwo = new DComboBox;
    m_consultationTypeTwo->setFixedSize(160, 36);
    m_consultationTypeTwo->setFont(archFont);
    m_consultationTypeTwo->setFocusPolicy(Qt::NoFocus);
    comboBoxLayout->addSpacing(13);
    comboBoxLayout->addWidget(m_consultationTypeTwo);

    m_consultationTypeThree = new DComboBox;
    m_consultationTypeThree->setFixedSize(160, 36);
    m_consultationTypeThree->setFont(archFont);
    m_consultationTypeThree->setFocusPolicy(Qt::NoFocus);
    comboBoxLayout->addSpacing(13);
    comboBoxLayout->addWidget(m_consultationTypeThree);
    comboBoxLayout->addSpacing(8);

    archFrame->setLayout(comboBoxLayout);

    return archFrame;
}

//初始化combo列表值
void ContactUs::initComboBox()
{
    //初始化system combobox
    m_consultationTypeOne->addItems(Devicetree::getInstance()->getSystemNames());
    //设置system combox 默认值为第一项，联动出发信号改变
    m_consultationTypeOne->setCurrentIndex(0);
}

void ContactUs::updateComboBox()
{
    m_consultationTypeOne->clear();
    m_consultationTypeOne->addItems(Devicetree::getInstance()->getSystemNames());
    m_consultationTypeOne->setCurrentIndex(0);
}

//服务端返回客户信息
void ContactUs::updateCustomerInfo(const QByteArray &data)
{
    QJsonParseError jsonError;
    QJsonDocument jsonDoc(QJsonDocument::fromJson(data, &jsonError));

    QJsonObject jsonObject = jsonDoc.object();

    if (jsonObject.isEmpty())
        return;

    ServiceCustomer customer;

    QString name = jsonObject["name"].toString();
    QString team = jsonObject["team"].toString();
    QString telephone = jsonObject["moblie"].toString();
    QString email = jsonObject["email"].toString();
    QString QRCode = jsonObject["qrcode"].toString();
    qDebug() << "updateCustomerInfo QRCode" << QRCode;

    if (!QRCode.isEmpty()) {
        QString qcodeFile = Devicetree::getInstance()->saveQRCode(QRCode);
        qDebug() << "qcodefile" << qcodeFile;

        //保存文件
        QFile file(qcodeFile);

        if (!file.open(QIODevice::ReadOnly)) {
            qDebug() << " QRCode File open error";
            return;
        } else {
            qDebug() << "QRCode File open!";
        }

        if (!QRCode.isEmpty()) {
            QImage qrcImage;
            if (qrcImage.load(qcodeFile)) {
                qDebug() << "load success";
            } else {
                qDebug() << "load failed";
                file.close();
                return;
            }

            QImage autoImage = qrcImage.scaled(m_qrcodeLabel->size(), Qt::IgnoreAspectRatio);
            m_qrcodeLabel->setPixmap(QPixmap::fromImage(autoImage));
        }

        file.close();
    }

    Devicetree::getInstance()->insertCustomerInfo(m_curSerchKey, customer);
}

void ContactUs::updateOfflineCustomInfo()
{
    SeviceCustomer customer = Devicetree::getInstance()->getCustomService(m_curSerchKey);
    qDebug() << "custpmer" << customer.QRCode;

    QString name = customer.name;
    QString team = customer.team;
    QString telephone = customer.telephone;
    QString email = customer.email;
    QString QRCode = customer.QRCode.split("/").last();

    QString Dir = "/usr/share/" + QCoreApplication::applicationName() + "/ini/" + "QRCode";
    QFileInfo info(QRCode);
    QString saveName = info.fileName();
    QString strFileName = Dir.append("/").append(saveName);
    QFile file(strFileName);

    if (!file.open(QIODevice::ReadOnly)) {
        qDebug() << "QRCode File open error";
        return;
    } else {
        qDebug() << "QRCode File open!";
    }

    if (!QRCode.isEmpty()) {
        QImage qrcImage;
        if (qrcImage.load(strFileName)) {
            qDebug() << "load success";
        } else {
            qDebug() << "load failed";
            file.close();
            return;
        }

        QImage autoImage = qrcImage.scaled(m_qrcodeLabel->size(), Qt::IgnoreAspectRatio);
        m_qrcodeLabel->setPixmap(QPixmap::fromImage(autoImage));
    }

    file.close();
}

//system combox 变化槽函数
void ContactUs::onSystemComboBoxIndexChanged(const QString &value)
{
    QStringList systemChangedList = Devicetree::getInstance()->getBlockNames(value);
    //联动改变block combox 内容
    m_consultationTypeTwo->clear();
    m_consultationTypeTwo->addItems(systemChangedList);
    m_consultationTypeTwo->setCurrentIndex(0);
}

//block combox 变化槽函数
void ContactUs::onBlockComboBoxIndexChanged(const QString &value)
{
    QStringList blockChangedList = Devicetree::getInstance()->getItemNames(value);
    //联动改变最后面的槽函数
    m_consultationTypeThree->clear();
    m_consultationTypeThree->addItems(blockChangedList);
    m_consultationTypeThree->setCurrentIndex(0);
}

void ContactUs::onItemComboBoxIndexChanged(const QString &value)
{
    //修改完combobox，发送消息到服务器读取企业客服信息
    QString systemComboBox = m_consultationTypeOne->currentText();
    QString blockComboBox = m_consultationTypeTwo->currentText();
    searchCustomerInfo(systemComboBox, blockComboBox, value);
}

//请求查找客服信息，读取本地配置文件进行加载
void ContactUs::searchCustomerInfo(const QString &system, const QString &block, const QString &item)
{
    QJsonObject rootObject;
    rootObject.insert("system", system);
    rootObject.insert("block", block);
    rootObject.insert("item", item);

    m_curSerchKey = system + "-" + block + "-" + item;

    Devicetree::getInstance()->loadCustomJsonFile();
    updateOfflineCustomInfo();
}
