/*
 * Copyright (C) 2019 ~ 2020 Uniontech Software Technology Co.,Ltd.
 *
 * Author:     like <like@uniontech.com>
 *
 * Maintainer: like <like@uniontech.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "deallogpthread.h"

#include <QDir>
#include <QDate>
#include <QFileInfo>
#include <QStandardPaths>

const QByteArray secretKey = "uos-server-support";

bool sortTime(QString dateOne, QString dateTwo)
{
    if (QDate::fromString(dateOne, "yyyy.MM.dd") > QDate::fromString(dateTwo, "yyyy.MM.dd"))
        return true;
    return false;
}

DealLogPthread::DealLogPthread(QObject *parent)
    : QThread(parent)
{
}

void DealLogPthread::run()
{
    QByteArray fileContent;
    QList<QString> fileNameList;
    QString readPath = QStandardPaths::writableLocation(QStandardPaths::AppLocalDataLocation) + "/log/";
    QDir readDir(readPath);
    if (!readDir.exists())
        return;

    QFileInfoList infoList = readDir.entryInfoList();
    foreach (QFileInfo fileInfo, infoList) {
        QString fileExtension = fileInfo.suffix();
        if (fileExtension != "log")
            continue;
        fileNameList.push_back(fileInfo.completeBaseName());
    }
    qSort(fileNameList.begin(), fileNameList.end(), sortTime);

    QString savePath = QStandardPaths::writableLocation(QStandardPaths::AppLocalDataLocation) + "/chat/";
    QDir saveDir(savePath);
    if (!saveDir.exists())
        saveDir.mkdir(savePath);

    foreach (QString fileName, fileNameList) {
        QFile file(readPath + fileName + ".log");
        QFile newFile(savePath + fileName + ".msg");

        if (!file.open(QFile::ReadOnly))
            continue;

        if (!newFile.open(QIODevice::WriteOnly | QIODevice::Text | QIODevice::Append))
            continue;
        while (!file.atEnd()) {
            fileContent = file.readLine();
            QByteArray secretData = encryptionMessage(fileContent);
            secretData.append("\n");
            newFile.write(secretData);
        }

        file.close();
        file.remove();
        newFile.close();
    }

    readDir.removeRecursively();
}

QByteArray DealLogPthread::encryptionMessage(const QByteArray &data)
{
    QByteArray secretData;
    char temp;
    int len = data.length();
    for (int i = 0; i < len; ++i) {
        temp = data.at(i);
        for (int j = 0; j < secretKey.length(); ++j) {
            temp = temp ^ secretKey.at(j);
        }
        secretData.append(temp);
    }

    return secretData;
}
