/*
 * Copyright (C) 2019 ~ 2020 Uniontech Software Technology Co.,Ltd.
 *
 * Author:     zhanglei <zhanglei@uniontech.com>
 *
 * Maintainer: zhanglei <zhanglei@uniontech.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "messageconsultationwidget.h"
#include "common/devicetree.h"

#include <QDBusInterface>
#include <QDebug>
#include <QMap>
#include <DFileDialog>
#include <QDialog>
#include <QCoreApplication>
#include <QMessageBox>
#include <QFileIconProvider>
#include <QTemporaryFile>
#include <QIcon>
#include <QJsonObject>
#include <QJsonDocument>
#include <QDBusReply>
#include <QDBusInterface>
#include <QStandardPaths>
#include <QApplication>
#include <QFileInfo>
#include <QRegExp>
#include <QToolTip>
#include <QProcess>

#include "dsysinfo.h"
#include <systemd/sd-journal.h>

#include <stdio.h>
#include <string.h>

#include <cstdlib>

DCORE_USE_NAMESPACE

//单个附件的大小不超过10M
const qint64 FILESIZE = 10 * 1024 * 1024;

EnclosureFrame::EnclosureFrame(QString fileName, DFrame *parent)
    : DFrame(parent)
{
    QHBoxLayout *horizontalLayout = new QHBoxLayout;
    QVBoxLayout *verticalLayout = new QVBoxLayout;

    QFont fileNameFont;
    fileNameFont.setPointSize(9);

    m_fileName = fileName;
    QFileInfo fileInfo(fileName);
    QFileIconProvider iconProvider;
    QIcon icon = iconProvider.icon(fileInfo);
    m_nameLabel = new DLabel;
    DLabel *sizeLabel = new DLabel;
    m_nameLabel->setFont(fileNameFont);
    sizeLabel->setFont(fileNameFont);
    DLabel *pIconLabel = new DLabel;
    pIconLabel->setFixedSize(26, 26);
    pIconLabel->setPixmap(icon.pixmap(26, 26));
    pIconLabel->setScaledContents(true);
    m_totalFileName = fileInfo.fileName();
    QString strElidedText = m_nameLabel->fontMetrics().elidedText(fileInfo.fileName(), Qt::ElideMiddle, 130, Qt::TextShowMnemonic);
//    qDebug() << "strElidedText" << strElidedText;
    m_nameLabel->setText(strElidedText);
    //计算文件名提示折行
    wrapFileName(m_totalFileName);
    m_closeLabel = new DLabel(this);
    QIcon iconClose(":icons/deepin/builtin/light/icons/close.svg");
    m_closeLabel->setPixmap(iconClose.pixmap(QSize(20, 20)));
    qint64 fileSize = fileInfo.size();
    QString fileSizeAndUnit;
    QString unit = "B";

    if (fileSize > 1024) {
        fileSize /= 1024;
        unit = "Kb";
        if (fileSize > 1024) {
            fileSize /= 1024;
            unit = "Mb";
            if (fileSize > 1024) {
                fileSize /= 1024;
                unit = "Gb";
            }
        }
    }

    fileSizeAndUnit.append(QString::number(fileSize)).append(unit);
    sizeLabel->setText(fileSizeAndUnit);
    verticalLayout->addWidget(m_nameLabel);
    verticalLayout->setSpacing(2);
    verticalLayout->addWidget(sizeLabel);
    horizontalLayout->addWidget(pIconLabel);
    horizontalLayout->addLayout(verticalLayout);
    horizontalLayout->addStrut(5);
    horizontalLayout->addWidget(m_closeLabel);
    horizontalLayout->setStretchFactor(pIconLabel, 2);
    horizontalLayout->setStretchFactor(verticalLayout, 7);
    horizontalLayout->setStretchFactor(m_closeLabel, 1);
    horizontalLayout->setSpacing(10);
    horizontalLayout->setContentsMargins(11, 5, 11, 5);
    this->setLayout(horizontalLayout);
    this->setBackgroundRole(DPalette::ItemBackground);
    this->setFixedSize(251, 48);
    m_nameLabel->installEventFilter(this);
    m_closeLabel->installEventFilter(this);
}

EnclosureFrame::~EnclosureFrame()
{
}

void EnclosureFrame::wrapFileName(const QString &fileName)
{
    QFont fontType;
    fontType.setPointSize(10);
    QFontMetrics fontMetrics(fontType);

    QString temp;
    for (int i = 0; i < fileName.length(); ++i) {
        temp += fileName.at(i);
        if (fontMetrics.width(temp) > 251) {
            temp.chop(1);
            m_promptFileName += temp + "\n";
            temp.clear();
            i--;
        }
    }
    if (!temp.isEmpty()) {
        m_promptFileName += temp;
    }
}

bool EnclosureFrame::eventFilter(QObject *object, QEvent *event)
{
    if (object == m_closeLabel) {
        if (event->type() == QEvent::MouseButtonRelease) {
            QMouseEvent *mouseEvent = dynamic_cast<QMouseEvent *>(event);
            if (mouseEvent != Q_NULLPTR) {
                if (mouseEvent->button() == Qt::LeftButton) {
                    emit updateFrameInfo(this, m_fileName);
                }
            }
        }
    }

    if (object == m_nameLabel) {
        if (event->type() == QEvent::Enter) {
            this->setToolTip(m_promptFileName);
        }
    }

    return DFrame::eventFilter(object, event);
}

MessageConsultation::MessageConsultation(QWidget *parent)
    : DWidget(parent)
{
    initUI();
    this->setAcceptDrops(true);
    connect(m_addEnclosureCmdlnkbtn, &DCommandLinkButton::clicked, this, &MessageConsultation::addEnclosureResp);

    //open log
    connect(m_logSwitchBtn, &DSwitchButton::clicked, this, &MessageConsultation::logSwitchBtnResponse);
    connect(m_mailLineEdit, &DLineEdit::textChanged, this, [ = ]() {
        m_mailLineEdit->setAlert(false);
    });
    connect(m_commitSuggestBtn, &DSuggestButton::clicked, this, &MessageConsultation::commitSuggestBtnResponse);
    connect(m_resetPushBtn, &DPushButton::clicked, this, [ = ]() {
        clearSendContent();
    });

    m_upLoadExplainTimer = new QTimer;
    connect(m_upLoadExplainTimer, &QTimer::timeout, this, [ = ]() {
        m_uploadexplainLabel->setHidden(true);
        m_upLoadExplainTimer->stop();
    });

    //初始化system combobox
    m_consultationTypeOne->addItems(Devicetree::getInstance()->getSystemNames());
    //设置system combox 默认值为第一项，联动出发信号改变
    m_consultationTypeOne->setCurrentIndex(0);
}

void MessageConsultation::commitSuggestBtnResponse()
{
    QString consultationTypeOne = m_consultationTypeOne->currentText();
    QString consultationTypeTwo = m_consultationTypeTwo->currentText();
    QString consultationTypeThree = m_consultationTypeThree->currentText();
    QString title = m_titleLineEdit->text();
    QString content = m_contentTextEdit->toPlainText();
    QString email = m_mailLineEdit->text();

    if (title.trimmed().isEmpty() && content.trimmed().isEmpty() && email.trimmed().isEmpty()) {
        popDialog(tr("The title, content and email are required, please fill in and submit it again"));
        return;
    }
    if (title.trimmed().isEmpty() && content.trimmed().isEmpty() && (email.trimmed().isEmpty() == false)) {
        popDialog(tr("The title and content are required, please fill in and submit it again"));
        return;
    }
    if ((content.trimmed().isEmpty()) && email.trimmed().isEmpty() && (title.trimmed().isEmpty() == false)) {
        popDialog(tr("The content and email are required, please fill in and submit it again"));
        return;
    }
    if ((title.trimmed().isEmpty()) && email.trimmed().isEmpty() && (content.trimmed().isEmpty() == false)) {
        popDialog(tr("The title and email are required, please fill in and submit it again"));
        return;
    }
    if ((title.trimmed().isEmpty()) && (content.trimmed().isEmpty() == false) && (email.trimmed().isEmpty() == false)) {
        popDialog(tr("Title are required, please fill in and submit again"));
        return;
    }
    if ((content.trimmed().isEmpty()) && (title.trimmed().isEmpty() == false) && (email.trimmed().isEmpty() == false)) {
        popDialog(tr("Content are required, please fill in and submit again"));
        return;
    }
    if ((email.trimmed().isEmpty()) && (title.trimmed().isEmpty() == false) && (content.trimmed().isEmpty() == false)) {
        popDialog(tr("Email are required, please fill in and submit again"));
        return;
    }

    //邮箱正则对于.的次数限制
    QRegExp patt("^(?!.*@.*@)[0-9a-z][_.0-9a-z-]{0,31}@([0-9a-z][0-9a-z]{0,30}[0-9a-z]){1,4}[\\.]{1}[a-z]{2,4}$");

    if (!patt.exactMatch(m_mailLineEdit->text())) {
        m_mailLineEdit->setAlert(true);
        m_mailLineEdit->showAlertMessage(tr("The email address is incorrect"), m_commitSuggestBtn, 3000);
        return;
    }

    //如果没有网，直接跳转页面
    if (!DNetState::getInstance()->getNetState()) {
        popDialog(tr("Network exception, please check the network before submitting"));
        return;
    }

    // 构建 Json 对象,内容为创建工单的基础信息
    QJsonObject tickerObj;
    tickerObj.insert("subject", title);
    tickerObj.insert("content", content);
    tickerObj.insert("creator_email", email);

    QJsonObject categoryObj;
    categoryObj.insert("system", consultationTypeOne);
    categoryObj.insert("block", consultationTypeTwo);
    categoryObj.insert("item", consultationTypeThree);

    emit lockConsultation();
    //发送数据
    emit signalTicket(getUserName(), QJsonDocument(tickerObj).toJson().simplified(), QJsonDocument(categoryObj).toJson().simplified(), m_filePathLists);
}

void MessageConsultation::postFailed()
{
    this->setAttribute(Qt::WA_TransparentForMouseEvents, false);
    popDialog(tr("The server connection is abnormal. Please try again later"));
}

void MessageConsultation::reGeometryLayout()
{
    QList<QLayoutItem *> items;

    for (int i = 0; i < m_enclosureGridLayout->count(); i++) {
        QLayoutItem *item = m_enclosureGridLayout->itemAt(i);
        items.push_back(item);
    }

    for (int i = 0; i < m_enclosureGridLayout->count(); i++) {
        QLayoutItem *item = m_enclosureGridLayout->itemAt(i);
        m_enclosureGridLayout->removeItem(item);
    }
    //grid layout 重新布局
    for (int i = 0; i < items.size(); i++) {
        int row = i / 2;
        int column = i % 2;
        //setParent为NULL，防止删除之后界面不消失
        if (items.at(i)->widget()) {
            //布局中添加控件
            m_enclosureGridLayout->addWidget(items.at(i)->widget(), row, column);
        }
    }
}

void MessageConsultation::deleteFrame(EnclosureFrame *enclosureFrame, const QString &fileName)
{
    if (!m_uploadexplainLabel->text().isEmpty()) {
        m_uploadexplainLabel->clear();
        m_uploadexplainLabel->setHidden(true);
    }

    if (enclosureFrame != Q_NULLPTR) {
        m_enclosureGridLayout->removeWidget(enclosureFrame);
        m_filePathLists.removeOne(fileName);
        enclosureFrame->deleteLater();
    }

    if (m_filePathLists.isEmpty() == true)
        m_horizontalLinethree->setHidden(true);

    reGeometryLayout();
}

void MessageConsultation::updateProgress(const QString &progress)
{
    if ((progress == "1") || (progress == "nan")) {
        m_commitSuggestBtn->setEnabled(true);
        m_resetPushBtn->setEnabled(true);
    } else {
        m_commitSuggestBtn->setEnabled(false);
        m_resetPushBtn->setEnabled(false);
    }
}

void MessageConsultation::ticketComplete(qint64 tickId)
{
    QFont fontSize;
    fontSize.setPointSize(9);
    this->setAttribute(Qt::WA_TransparentForMouseEvents, false);
    qDebug() << "ticketComplete tikID" << tickId;
    if (tickId != 0) {
        clearSendContent();
    } else {
        popDialog(tr("The server connection is abnormal. Please try again later"));
    }
}

void MessageConsultation::clearSendContent()
{
    m_titleLineEdit->clear();
    m_contentTextEdit->clear();
    m_mailLineEdit->clear();
    m_uploadexplainLabel->clear();
    m_uploadexplainLabel->setHidden(true);
    m_logSwitchBtn->setChecked(false);
    m_horizontalLinethree->setHidden(true);
    updateComboBox();

    //清除附件列表
    m_filePathLists.clear();
    //清空附件附件布局中的所有元素
    QLayoutItem *child;

    while ((child = m_enclosureGridLayout->takeAt(0)) != Q_NULLPTR) {
        //setParent为NULL，防止删除之后界面不消失
        if (child->widget() != Q_NULLPTR) {
            child->widget()->setParent(Q_NULLPTR);
        }
        delete child;
        child = Q_NULLPTR;
    }
}

MessageConsultation::~MessageConsultation()
{
}

int MessageConsultation::readLog()
{
    QString oldPath = QDir::currentPath();
    QString saveName = "user_report.log";
    QString savePath = QStandardPaths::writableLocation(QStandardPaths::AppLocalDataLocation) + "/user_collect_log/";
    QDir logDir;

    if (!logDir.exists(savePath)) {
        logDir.mkpath(savePath);
    }

    //将程序的执行路径设置到filePath下
    logDir.setCurrent(savePath);
    QFile writeFile(saveName);

    if (!writeFile.open(QIODevice::WriteOnly | QIODevice::Truncate)) {
        qDebug() << tr("open file fail");
        return OpenFileFailed;
    }

    int result;
    sd_journal *journal;
    result = sd_journal_open(&journal, SD_JOURNAL_CURRENT_USER | SD_JOURNAL_SYSTEM);
    if (result < 0) {
        fprintf(stderr, "Failed to open journal: %s\n", strerror(-result));
        logDir.setCurrent(oldPath);
        return OpenFileFailed;
    }

    uint64_t from = static_cast<uint64_t>(QDateTime::currentDateTime().addDays(-2).toMSecsSinceEpoch());
    uint64_t to = static_cast<uint64_t>(QDateTime::currentMSecsSinceEpoch());

    result = sd_journal_get_cutoff_realtime_usec(journal, &from, &to);
    SD_JOURNAL_FOREACH(journal)
    {
        const char *data;
        size_t size;
        result = sd_journal_get_data(journal, "MESSAGE", reinterpret_cast<const void **>(&data), &size);

        if (result < 0) {
            fprintf(stderr, "Failed to read message field: %s\n", strerror(-result));
            logDir.setCurrent(oldPath);
            continue;
        }

        if (m_logSwitchBtn->isChecked()) {
            writeFile.write(data);
        } else {
            sd_journal_close(journal);
            writeFile.close();
            logDir.setCurrent(oldPath);
            return UpLoadRevoke;
        }
        QApplication::processEvents();
    }

    sd_journal_close(journal);
    writeFile.close();
    QProcess::startDetached("zip -r -j " + savePath + "user_report_log.zip " + savePath);
    logDir.setCurrent(oldPath);
    return Success;
}

void MessageConsultation::popDialog(const QString &message)
{
    DDialog promptDialog(this);
    promptDialog.addButton(tr("OK"));
    promptDialog.setIcon(QIcon::fromTheme("uss_warning"));
    promptDialog.setMessage(message);
    promptDialog.exec();
}

void MessageConsultation::initUI()
{
    QFont fontSize;
    fontSize.setPointSize(10);
    QScrollArea *scrollArea = new QScrollArea(this);
    scrollArea->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    QWidget *scrollAreaWidgetContents = new QWidget;
    scrollArea->setWidget(scrollAreaWidgetContents);

    QVBoxLayout *mainLayout = new QVBoxLayout;
    QVBoxLayout *scrollVBoxLayout = new QVBoxLayout;

    DFrame *consultationTopFrame = new DFrame;
    DFrame *horizontalLineOne = new DFrame(consultationTopFrame); //使用frame实现分割线功能
    horizontalLineOne->setLineWidth(0);
    horizontalLineOne->setFixedSize(620, 1);
    horizontalLineOne->move(-5, 47);

    DFrame *horizontalLineTwo = new DFrame(consultationTopFrame);
    horizontalLineTwo->setLineWidth(0);
    horizontalLineTwo->setFixedSize(620, 1);
    horizontalLineTwo->move(-5, 94);
    m_horizontalLinethree = new DFrame(this);
    m_horizontalLinethree->setLineWidth(0);
    m_horizontalLinethree->setFixedSize(514, 1);
    m_horizontalLinethree->setBackgroundRole(DPalette::ItemBackground);
    m_horizontalLinethree->setHidden(true);

    consultationTopFrame->setLineWidth(0);
    consultationTopFrame->setFixedWidth(600);
    consultationTopFrame->setAcceptDrops(true);
    consultationTopFrame->setBackgroundRole(DPalette::ItemBackground);
    QVBoxLayout *consultationTopLayout = new QVBoxLayout;
    consultationTopLayout->setContentsMargins(0, 0, 0, 0);

    QHBoxLayout *consultationTypeBoxLayout = new QHBoxLayout;
    consultationTypeBoxLayout->setContentsMargins(10, 0, 8, 0);

    DLabel *consultationTypeLabel = new DLabel;
    consultationTypeLabel->setText(tr("Category"));
    consultationTypeLabel->setFont(fontSize);

    m_consultationTypeOne = new DComboBox;
    m_consultationTypeOne->setFont(fontSize);
    m_consultationTypeOne->setFocusPolicy(Qt::NoFocus);
    m_consultationTypeOne->setFixedSize(164, 36);
    m_consultationTypeTwo = new DComboBox;
    m_consultationTypeTwo->setFont(fontSize);
    m_consultationTypeTwo->setFixedSize(164, 36);
    m_consultationTypeTwo->setFocusPolicy(Qt::NoFocus);
    m_consultationTypeThree = new DComboBox;
    m_consultationTypeThree->setFont(fontSize);
    m_consultationTypeThree->setFixedSize(164, 36);
    m_consultationTypeThree->setFocusPolicy(Qt::NoFocus);

    //信号槽链接
    QObject::connect(m_consultationTypeOne, &QComboBox::currentTextChanged, this, &MessageConsultation::onSystemComboBoxIndexChanged);
    QObject::connect(m_consultationTypeTwo, &QComboBox::currentTextChanged, this, &MessageConsultation::onBlockComboBoxIndexChanged);
    QObject::connect(m_consultationTypeThree, &QComboBox::currentTextChanged, this, &MessageConsultation::onItemComboBoxIndexChanged);

    consultationTypeBoxLayout->addWidget(consultationTypeLabel);
    consultationTypeBoxLayout->addStretch();
    consultationTypeBoxLayout->addWidget(m_consultationTypeOne);
    consultationTypeBoxLayout->addSpacing(4);
    consultationTypeBoxLayout->addWidget(m_consultationTypeTwo);
    consultationTypeBoxLayout->addSpacing(4);
    consultationTypeBoxLayout->addWidget(m_consultationTypeThree);

    QHBoxLayout *titleHBoxLayout = new QHBoxLayout;
    titleHBoxLayout->setContentsMargins(10, 0, 8, 0);
    DLabel *titleLabel = new DLabel;
    titleLabel->setText(tr("Title"));
    titleLabel->setFont(fontSize);

    m_titleLineEdit = new DLineEdit;
    m_titleLineEdit->lineEdit()->setAcceptDrops(false);
    m_titleLineEdit->setFixedSize(512, 36);
    m_titleLineEdit->lineEdit()->setPlaceholderText(tr("(Required) Input the title you want to consult or suggest"));
    m_titleLineEdit->setFont(fontSize);
    m_titleLineEdit->lineEdit()->setMaxLength(31);

    connect(m_titleLineEdit, &DLineEdit::textChanged, this,
            [ = ](const QString &keyText) {
                if (keyText.size() == 31) {
                    QString text = keyText.left(keyText.length() - 1);
                    m_titleLineEdit->setText(text);
                    popDialog(tr("The title should be no more than 30 characters"));
                    return;
                }
            });
    titleHBoxLayout->addWidget(titleLabel);
    titleHBoxLayout->addStretch();
    titleHBoxLayout->addWidget(m_titleLineEdit);

    QVBoxLayout *contentAllLayout = new QVBoxLayout;
    contentAllLayout->setContentsMargins(10, 3, 8, 0);
    QHBoxLayout *contentOneLayout = new QHBoxLayout;
    contentOneLayout->setContentsMargins(0, 0, 0, 0);

    DLabel *contentLabel = new DLabel(tr("Content"));
    contentLabel->setFont(fontSize);
    m_contentTextEdit = new DTextEdit;
    m_contentTextEdit->setPlaceholderText(tr("(Required)\n "
                                             "\n"
                                             "* Please write the detailed description\n"
                                             "\n"
                                             "* You can attach screenshots, videos, and GIFs (using Screen Capture)"));
    m_contentTextEdit->setFont(fontSize);
    m_contentTextEdit->setAcceptDrops(false);
    m_contentTextEdit->setFixedSize(512, 170);

    contentOneLayout->addWidget(contentLabel, 0, Qt::AlignTop);
    contentOneLayout->addWidget(m_contentTextEdit, 0, Qt::AlignTop);

    DPalette pal;
    pal.setColor(DPalette::TextTips, QColor());

    QHBoxLayout *contentTwoLayout = new QHBoxLayout;
    contentTwoLayout->setContentsMargins(0, 0, 0, 0);
    m_contentCountLabel = new DLabel("0/200", m_contentTextEdit);
    m_contentCountLabel->setPalette(pal);
    m_contentCountLabel->setFixedSize(50, 18);
    m_contentCountLabel->setAlignment(Qt::AlignRight);
    m_contentCountLabel->move(450, 140);
    m_contentCountLabel->setFont(fontSize);
    m_addEnclosureCmdlnkbtn = new DCommandLinkButton(tr("text"));
    DLabel *enclosureSizeLabel = new DLabel(tr("(Each one ≤ 10 MB)"));
    enclosureSizeLabel->setFont(fontSize);
    enclosureSizeLabel->setPalette(pal);
    m_addEnclosureCmdlnkbtn->setText(tr("Add attachments "));
    m_addEnclosureCmdlnkbtn->setFont(fontSize);
    m_addEnclosureCmdlnkbtn->setFocusPolicy(Qt::NoFocus);

    contentTwoLayout->addStretch();
    contentTwoLayout->addWidget(m_addEnclosureCmdlnkbtn);
    contentTwoLayout->addWidget(enclosureSizeLabel);

    connect(m_contentTextEdit, &QTextEdit::textChanged, this, [ = ]() {
        QString textContent = m_contentTextEdit->toPlainText();
        int length = textContent.count();
        int maxLength = 200; // 内容区最大字符数
        QString num = QString::number(length);
        m_contentCountLabel->setText(num + "/200");
        m_contentCountLabel->setFont(fontSize);
        if (length > maxLength) {
            int position = m_contentTextEdit->textCursor().position();
            QTextCursor textCursor = m_contentTextEdit->textCursor();
            textContent.remove(position - (length - maxLength), length - maxLength);
            m_contentTextEdit->setText(textContent);
            textCursor.setPosition(position - (length - maxLength));
            m_contentTextEdit->setTextCursor(textCursor);
        }
    });

    m_uploadexplainLabel = new DLabel;
    m_uploadexplainLabel->setStyleSheet("color:#FF5736;");
    m_uploadexplainLabel->setText(tr(""));
    m_uploadexplainLabel->setHidden(true);

    m_enclosureGridLayout = new QGridLayout;
    m_enclosureGridLayout->setHorizontalSpacing(10);
    m_enclosureGridLayout->setVerticalSpacing(10);
    m_enclosureGridLayout->setContentsMargins(0, 5, 0, 10);
    m_enclosureGridLayout->setAlignment(Qt::AlignRight | Qt::AlignTop);

    contentAllLayout->addLayout(contentOneLayout);
    contentAllLayout->addLayout(contentTwoLayout);
    contentAllLayout->addWidget(m_horizontalLinethree, 0, Qt::AlignRight);
    contentAllLayout->addLayout(m_enclosureGridLayout);
    contentAllLayout->addWidget(m_uploadexplainLabel, 0, Qt::AlignRight);

    consultationTopLayout->addSpacing(4);
    consultationTopLayout->addLayout(consultationTypeBoxLayout);
    consultationTopLayout->addSpacing(7);
    consultationTopLayout->addLayout(titleHBoxLayout);
    consultationTopLayout->addSpacing(7);
    consultationTopLayout->addLayout(contentAllLayout);
    consultationTopFrame->setLayout(consultationTopLayout);

    m_logLabelHBoxLayout = new QHBoxLayout;

    DFrame *logFrame = new DFrame;
    logFrame->setLineWidth(0);
    logFrame->setBackgroundRole(DPalette::ItemBackground);
    logFrame->setFixedSize(600, 48);

    DLabel *logLabel = new DLabel(tr("Upload log"));
    logLabel->setFont(fontSize);

    QLabel *logExplainLabel = new QLabel;
    QFont logFont;
    logFont.setPointSize(8);
    QString str = tr("(Your personal information will not be collected when uploading logs)");
    logExplainLabel->setFont(logFont);
    logExplainLabel->setText(str);
    logExplainLabel->setFixedHeight(18);
    logExplainLabel->setPalette(pal);

    m_logSwitchBtn = new DSwitchButton;
    m_logSwitchBtn->setFocusPolicy(Qt::NoFocus);
    m_logLabelHBoxLayout->setContentsMargins(9, 0, 6, 0);
    m_logLabelHBoxLayout->addWidget(logLabel);
    m_logLabelHBoxLayout->addWidget(logExplainLabel);
    m_logLabelHBoxLayout->addStretch();
    m_logLabelHBoxLayout->addWidget(m_logSwitchBtn);
    logFrame->setLayout(m_logLabelHBoxLayout);

    DFrame *mailFrame = new DFrame;
    mailFrame->setFixedSize(600, 48);
    mailFrame->setLineWidth(0);
    mailFrame->setBackgroundRole(DPalette::ItemBackground);

    QHBoxLayout *mailHBoxLayout = new QHBoxLayout;
    mailHBoxLayout->setContentsMargins(10, 0, 8, 0);
    DLabel *mailLabel = new DLabel(tr("E-mail"));
    mailLabel->setFont(fontSize);

    m_mailLineEdit = new DLineEdit;
    m_mailLineEdit->setFixedSize(512, 36);
    m_mailLineEdit->lineEdit()->setAcceptDrops(false);

    mailHBoxLayout->addWidget(mailLabel);
    mailHBoxLayout->addStretch();
    mailHBoxLayout->addWidget(m_mailLineEdit);
    mailFrame->setLayout(mailHBoxLayout);

    m_mailLineEdit->lineEdit()->setPlaceholderText(tr("(Required) Write down your email to get the progress timely"));
    m_mailLineEdit->setClearButtonEnabled(true);
    m_mailLineEdit->setFont(fontSize);

    QHBoxLayout *buttonHLayout = new QHBoxLayout;
    m_commitSuggestBtn = new DSuggestButton(tr("Submit"));
    m_commitSuggestBtn->setFont(fontSize);
    m_commitSuggestBtn->setFocusPolicy(Qt::NoFocus);
    m_resetPushBtn = new DPushButton(tr("Reset"));
    m_resetPushBtn->setFont(fontSize);
    m_resetPushBtn->setFocusPolicy(Qt::NoFocus);
    m_commitSuggestBtn->setFixedSize(170, 36);
    m_resetPushBtn->setFixedSize(170, 36);
    buttonHLayout->addStretch(0);
    buttonHLayout->addWidget(m_resetPushBtn);
    buttonHLayout->addSpacing(14);
    buttonHLayout->addWidget(m_commitSuggestBtn);
    buttonHLayout->addSpacing(10);

    scrollVBoxLayout->addWidget(consultationTopFrame);
    scrollVBoxLayout->addSpacing(6);
    scrollVBoxLayout->addWidget(logFrame);
    scrollVBoxLayout->addSpacing(6);
    scrollVBoxLayout->addWidget(mailFrame);
    scrollVBoxLayout->addStretch();

    scrollArea->setFrameShape(QFrame::NoFrame);
    scrollArea->widget()->setLayout(scrollVBoxLayout);
    mainLayout->setMargin(1);
    mainLayout->addWidget(scrollArea);
    mainLayout->addLayout(buttonHLayout);
    mainLayout->addSpacing(8);

    scrollArea->setWidgetResizable(true);
    setLayout(mainLayout);
}

void MessageConsultation::addEnclosureResp()
{
    QFont fontSize;
    fontSize.setPointSize(9);
    DFileDialog fileDlg(this); //创建对话框
    //设置默认文件路径
    QString strDir = QDir::homePath();
    fileDlg.setDirectory(strDir);
    //设置可以选择多个文件,默认为只能选择一个文件QFileDialog::ExistingFilesCancel
    fileDlg.setFileMode(DFileDialog::ExistingFiles);
    //设置视图模式
    fileDlg.setViewMode(DFileDialog::Detail);

    QStringList fileNames;
    if (fileDlg.exec())
        fileNames = fileDlg.selectedFiles();

    //选择的文件中不能含有svg文件，否则添加失败并弹出警告
    for (auto file : fileNames) {
        if (file.split(".").last() == "svg") {
            popDialog(tr(".svg files are not supported"));
            return;
        }
    }

    int count = fileNames.size() + m_filePathLists.count();

    if (m_logSwitchBtn->isChecked())
        count--;

    if (count > 4) {
        popDialog(tr("Attachments ≤ 4 pcs"));
        return;
    }

    handleFile(fileNames);
}

void MessageConsultation::handleFile(const QStringList &files)
{
    QFont fontSize;
    fontSize.setPointSize(9);
    //再次添加清除label內容和销毁该布局
    if (!m_uploadexplainLabel->text().isEmpty()) {
        m_uploadexplainLabel->clear();
        m_uploadexplainLabel->setHidden(true);
    }

    int startCount = m_filePathLists.count();
    int j = 0;

    for (int i = 0; i < files.count(); i++) {
        QString file = files.at(i);
        QFileInfo info(file);

        if (info.size() > FILESIZE) {
            j++;
            continue;
        }

        if (info.size() == 0) {
            j++;
            continue;
        }

        EnclosureFrame *frameInfo = new EnclosureFrame(file);
        int count = startCount + i + 1 - j;
        connect(frameInfo, &EnclosureFrame::updateFrameInfo, this, &MessageConsultation::deleteFrame);

        if (m_logSwitchBtn->isChecked())
            count--;

        int row = (count - 1) / 2;
        int column = count % 2;

        if (m_horizontalLinethree->isHidden() == true)
            m_horizontalLinethree->setHidden(false);
        //布局中添加控件
        m_enclosureGridLayout->addWidget(frameInfo, row, column);
        reGeometryLayout();
        m_filePathLists.push_back(file);
    }

    for (int i = 0; i < files.count(); i++) {
        QString file = files.at(i);
        QFileInfo info(file);

        if (info.size() > FILESIZE) {
            m_uploadexplainLabel->setText(tr("Each one ≤ 10 MB"));
            m_uploadexplainLabel->setHidden(false);
            m_upLoadExplainTimer->start(3000);
        }

        if (info.size() == 0) {
            m_uploadexplainLabel->setText(tr("Attachments cannot be files with 0 bytes"));
            m_uploadexplainLabel->setHidden(false);
            m_upLoadExplainTimer->start(3000);
        }
    }
}

//屏蔽文件夹，不支持拖拽文件夹
void MessageConsultation::dragEnterEvent(QDragEnterEvent *event)
{
    if (event->mimeData()->hasFormat("text/uri-list")) {
        QList<QUrl> urls = event->mimeData()->urls();

        foreach (QUrl url, urls) {
            QFileInfo fileInfo(url.toLocalFile());
            if (fileInfo.isDir() || (fileInfo.fileName().split(".").last() == "svg")) {
                event->ignore();
                return;
            }
        }
        event->acceptProposedAction();
    } else {
        event->ignore();
    }
}

void MessageConsultation::dropEvent(QDropEvent *event)
{
    QList<QUrl> urls = event->mimeData()->urls();

    if (urls.isEmpty())
        return;

    int count = urls.size() + m_filePathLists.count();

    if (m_logSwitchBtn->isChecked())
        count--;

    if (count > 4) {
        popDialog(tr("Attachments ≤ 4 pcs"));
        return;
    }

    QStringList files;

    foreach (QUrl url, urls)
        files.push_back(url.toLocalFile());

    handleFile(files);
}

QStringList MessageConsultation::getFilePaths()
{
    return m_filePathLists;
}

QString MessageConsultation::getUserName()
{
    QDBusInterface manual("com.deepin.daemon.Accounts", "/com/deepin/daemon/Accounts/User1000", "com.deepin.daemon.Accounts.User", QDBusConnection::systemBus());
    QString name = manual.property("UserName").toString();
    return name;
}

void MessageConsultation::updateComboBox()
{
    m_consultationTypeOne->clear();
    m_consultationTypeOne->addItems(Devicetree::getInstance()->getSystemNames());
    m_consultationTypeOne->setCurrentIndex(0);
}

void MessageConsultation::onSystemComboBoxIndexChanged(const QString &value)
{
    QStringList list = Devicetree::getInstance()->getBlockNames(value);
    m_consultationTypeTwo->clear();
    m_consultationTypeTwo->addItems(list);
    m_consultationTypeTwo->setCurrentIndex(0);
}

void MessageConsultation::onBlockComboBoxIndexChanged(const QString &value)
{
    QStringList list = Devicetree::getInstance()->getItemNames(value);
    m_consultationTypeThree->clear();
    m_consultationTypeThree->addItems(list);
    m_consultationTypeThree->setCurrentIndex(0);
}

void MessageConsultation::onItemComboBoxIndexChanged(const QString &value)
{
    //修改完combobox，发送消息到服务器读取企业客服信息
}

void MessageConsultation::logSwitchBtnResponse(bool flag)
{
    QFont fontSize;
    fontSize.setPointSize(9);

    if (flag) {
        readLog();
        QString saveName = "user_report_log.zip";
        QString savePath = QStandardPaths::writableLocation(QStandardPaths::AppLocalDataLocation) + "/user_collect_log/";
        QString filePath = savePath + saveName;
        m_filePathLists.push_back(filePath);
    } else {
        QString savename = "user_report_log.zip";
        QString savepath = QStandardPaths::writableLocation(QStandardPaths::AppLocalDataLocation) + "/user_collect_log/";
        QString filepath = savepath + savename;
        m_filePathLists.removeOne(filepath);
    }
}
