/*
 * Copyright (C) 2019 ~ 2020 Uniontech Software Technology Co.,Ltd.
 *
 * Author:     zhanglei <zhanglei@uniontech.com>
 *
 * Maintainer: zhanglei <zhanglei@uniontech.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef MESSAGECONSULTATION_H
#define MESSAGECONSULTATION_H

#include "common/clientsocket.h"
#include "common/unit.h"

#include <DWidget>
#include <DFrame>
#include <DLineEdit>
#include <DTextEdit>
#include <DListView>
#include <DSwitchButton>
#include <DSpinner>
#include <DSuggestButton>
#include <DPushButton>
#include <DScrollArea>
#include <DListView>
#include <DCommandLinkButton>
#include <DDialog>
#include <DLabel>
#include <DComboBox>

#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QGridLayout>
#include <QStandardItemModel>
#include <QIcon>
#include <QFont>
#include <QDragEnterEvent>
#include <QDropEvent>
#include <QMimeData>
#include <QUrl>

DWIDGET_USE_NAMESPACE

class EnclosureFrame : public DFrame
{
    Q_OBJECT
public:
    EnclosureFrame(QString fileName, DFrame *parent = Q_NULLPTR);
    ~EnclosureFrame() override;
    void wrapFileName(const QString &fileName);

protected:
    bool eventFilter(QObject *object, QEvent *event) override;

private:
    DLabel *m_nameLabel;
    DLabel *m_closeLabel;
    QString m_fileName;
    QString m_totalFileName;
    QString m_promptFileName;

signals:
    void updateFrameInfo(EnclosureFrame *enclosureFrame, QString fileName);
};

class MessageConsultation : public DWidget
{
    Q_OBJECT
public:
    enum LogResult {
        //打开文件失败
        OpenFileFailed = -1,
        //提交中撤销
        UpLoadRevoke,
        //提交成功
        Success,
    };

    explicit MessageConsultation(QWidget *parent = Q_NULLPTR);
    ~MessageConsultation() override;

public slots:
    //设置故障树列表
    void updateComboBox();
    //工单创建成功
    void ticketComplete(qint64 tickId);
    //更新上传进度
    void updateProgress(const QString &progress);
    //post失败
    void postFailed();
    void deleteFrame(EnclosureFrame *enclosureFrame, const QString &fileName);

protected:
    void dragEnterEvent(QDragEnterEvent *event) override; //拖动进入事件
    void dropEvent(QDropEvent *event) override;

private:
    //combox 设置
    DComboBox *m_consultationTypeOne;
    DComboBox *m_consultationTypeTwo;
    DComboBox *m_consultationTypeThree;
    DLineEdit *m_titleLineEdit;
    DTextEdit *m_contentTextEdit;
    DLabel *m_contentCountLabel;
    DCommandLinkButton *m_addEnclosureCmdlnkbtn;
    QGridLayout *m_enclosureGridLayout;
    DLabel *m_uploadexplainLabel;
    QHBoxLayout *m_logHBoxLayout;
    QHBoxLayout *m_logLabelHBoxLayout;
    DSwitchButton *m_logSwitchBtn;
    QLabel *m_uploadLabel;
    DSpinner *m_logSpinner;
    DLabel *m_consultationImageLabel;
    DLineEdit *m_mailLineEdit;
    DSuggestButton *m_commitSuggestBtn;
    DPushButton *m_resetPushBtn;
    DFrame *m_horizontalLinethree;
    QTimer *m_upLoadExplainTimer;

    int m_length;
    //服务器连接状态
    bool m_bConnected;
    QStringList m_filePathLists;

    void handleFile(const QStringList &files);
    void reGeometryLayout();
    QStringList getFilePaths();
    QString getUserName();

    //重置和发送成功，清除发送的内容
    void clearSendContent();
    void initUI();
    int readLog();
    void popDialog(const QString &message);

private slots:
    // 服务器信息返回处理
    void addEnclosureResp();
    void onSystemComboBoxIndexChanged(const QString &value);
    void onBlockComboBoxIndexChanged(const QString &value);
    void onItemComboBoxIndexChanged(const QString &value);
    void logSwitchBtnResponse(bool flag);
    void commitSuggestBtnResponse();

signals:
    void signalTicket(const QString &userName, const QByteArray &ticketData, const QByteArray &categoryData, const QStringList &filePathLists);
    void ticketCreatSuccess();
    void lockConsultation();
};

#endif // MESSAGECONSULTATION_H
