/*
 * Copyright (C) 2019 ~ 2020 Uniontech Software Technology Co.,Ltd.
 *
 * Author:     liutianming <liutianming@uniontech.com>
 *
 * Maintainer: liutianming <liutianming@uniontech.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "nonetworkwidget.h"

#include <DIconButton>

#include <QVBoxLayout>

const int TIMERNUM = 5;

NoNetworkWidget::NoNetworkWidget(QWidget *parent)
    : DWidget(parent)
{
    m_timeNum = TIMERNUM;

    initUI();
    m_timer = new QTimer(this);
}

NoNetworkWidget::~NoNetworkWidget()
{
    m_timer->deleteLater();
}

void NoNetworkWidget::resetValue()
{
    m_timeNum = TIMERNUM;

    if (m_timer->isActive()) {
        m_timer->stop();
        m_timer->start(1000);
        m_countDown = QString(tr("(in ")) + QString("5s)");
        m_countDownLabel->setText(m_countDown);
    }
}

void NoNetworkWidget::initUI()
{
    this->setFixedSize(620, 580);

    QFont fontType;
    fontType.setPointSize(9);

    DIconButton *noNetworkImage = new DIconButton(this);
    noNetworkImage->setIcon(QIcon::fromTheme("uss_noNet"));
    noNetworkImage->setFlat(true);
    noNetworkImage->setFixedSize(100, 80);
    noNetworkImage->setIconSize(QSize(100, 80));
    noNetworkImage->setFocusPolicy(Qt::NoFocus);

    DLabel *noNetworkLabel = new DLabel;
    noNetworkLabel->setText(tr("No Network Connection"));
    QFont noNetworkFont("SourceHanSansSC-Bold,SourceHanSansSC", 12, 75);
    noNetworkLabel->setFont(noNetworkFont);

    m_contentLabel = new DLabel;
    m_content = tr("You may use internal network or network unavailable,");
    m_content.append(tr("\nand will be redirected to the Contact Us page.You also can seek help in other ways."));
    m_contentLabel->setFont(fontType);
    m_contentLabel->setText(m_content);
    m_contentLabel->setAlignment(Qt::AlignHCenter);

    m_countDownLabel = new DLabel;
    m_countDown.append(tr("(in ")).append("5").append("s)");
    m_countDownLabel->setFont(fontType);
    m_countDownLabel->setText(m_countDown);
    m_countDownLabel->setAlignment(Qt::AlignHCenter);

    QVBoxLayout *overallLayout = new QVBoxLayout;
    overallLayout->setContentsMargins(0, 0, 0, 0);
    overallLayout->addSpacing(132);
    overallLayout->addWidget(noNetworkImage, 0, Qt::AlignCenter);
    overallLayout->addSpacing(35);
    overallLayout->addWidget(noNetworkLabel, 0, Qt::AlignCenter);
    overallLayout->addWidget(m_contentLabel, 0, Qt::AlignCenter);
    overallLayout->addWidget(m_countDownLabel, 0, Qt::AlignCenter);
    overallLayout->addStretch();

    this->setLayout(overallLayout);
}

void NoNetworkWidget::setLabelText()
{
    QString time = QString("%1").arg(m_timeNum);
    m_countDown = QString(tr("(in ")) + QString(time) + QString("s)");
    m_countDownLabel->setText(m_countDown);
}

void NoNetworkWidget::showEvent(QShowEvent *event)
{
    m_countDown = QString(tr("(in ")) + QString("5s)");
    m_countDownLabel->setText(m_countDown);

    //定时启动
    m_timeNum = TIMERNUM;
    m_timer = new QTimer;
    connect(m_timer, &QTimer::timeout, this, &NoNetworkWidget::setCountdown);
    m_timer->start(1000);
}

void NoNetworkWidget::hideEvent(QHideEvent *event)
{
    m_timeNum = TIMERNUM;
    m_timer->stop();
}

void NoNetworkWidget::setCountdown()
{
    m_timeNum--;
    setLabelText();

    if (m_timeNum <= 0) {
        m_timeNum = TIMERNUM;
        m_timer->stop();

        emit changePage();
    }
}
