/*
 * Copyright (C) 2019 ~ 2020 Uniontech Software Technology Co.,Ltd.
 *
 * Author:     like <like@uniontech.com>
 *
 * Maintainer: like <like@uniontech.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "openhelpmanual.h"

#include <DIconButton>
#include <DLabel>
#include <DFontSizeManager>

#include <QEvent>
#include <QVBoxLayout>
#include <QDBusInterface>
#include <QDBusReply>
#include <QDebug>
#include <QProcess>

openHelpManual::openHelpManual(QWidget *parent)
    : DFrame(parent)
{
    this->setFixedSize(140, 140);
    this->setLineWidth(0);

    QFont fontType;
    fontType.setPointSize(9);

    DIconButton *helpManualIcon = new DIconButton(this);
    helpManualIcon->setIcon(QIcon::fromTheme("uss_help"));
    helpManualIcon->setFlat(true);
    helpManualIcon->setFocusPolicy(Qt::NoFocus);
    helpManualIcon->setFixedSize(96, 96);
    helpManualIcon->setIconSize(QSize(96, 96));

    DLabel *linkButton = new DLabel("text");
    linkButton->setFont(fontType);
    linkButton->setText(tr("Manual"));
    linkButton->setFont(fontType);

    QVBoxLayout *overallLayout = new QVBoxLayout;
    overallLayout->setContentsMargins(0, 0, 0, 0);
    overallLayout->addStretch();
    overallLayout->addWidget(helpManualIcon, 0, Qt::AlignCenter);
    overallLayout->addSpacing(14);
    overallLayout->addWidget(linkButton, 0, Qt::AlignCenter);
    overallLayout->addStretch();
    this->setLayout(overallLayout);

    this->installEventFilter(this);
    helpManualIcon->setAttribute(Qt::WA_TransparentForMouseEvents);
}

openHelpManual::~openHelpManual()
{
}

bool openHelpManual::eventFilter(QObject *object, QEvent *event)
{
    if (object == this) {
        if (event->type() == QEvent::MouseButtonRelease) {
            QDBusInterface manual("com.deepin.Manual.Open", "/com/deepin/Manual/Open", "com.deepin.Manual.Open", QDBusConnection::sessionBus());

            QDBusReply<void> reply = manual.call("Open", "uos-service-support");
            if (reply.isValid()) {
                qDebug() << "call com.deepin.Manual.Open success";
                return QFrame::eventFilter(object, event);
            }

            qDebug() << "call com.deepin.Manual.Open failed" << reply.error();
        }
    }

    return QFrame::eventFilter(object, event);
}
