/*
 * Copyright (C) 2019 ~ 2020 Uniontech Software Technology Co.,Ltd.
 *
 * Author:     zhangminjie <zhangminjie@uniontech.com>
 *
 * Maintainer: zhangminjie <zhangminjie@uniontech.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "selfsuppoetmodule.h"

#include <DIconButton>
#include <DLabel>

#include <QUrl>
#include <QEvent>
#include <QVBoxLayout>
#include <QDesktopServices>

SelfSupportModule::SelfSupportModule(QWidget *parent, const QString &iconName, const QString &linkName, const QString &linkAddress)
    : DFrame(parent)
    , m_iconName(iconName)
    , m_linkName(linkName)
    , m_linkAddress(linkAddress)
{
    this->setFixedSize(140, 140);
    this->setLineWidth(0);

    QFont fontType;
    fontType.setPointSize(9);

    DIconButton *titleIcon = new DIconButton(this);
    titleIcon->setIcon(QIcon::fromTheme(m_iconName));
    titleIcon->setFlat(true);
    titleIcon->setFocusPolicy(Qt::NoFocus);
    titleIcon->setFixedSize(96, 96);
    titleIcon->setIconSize(QSize(96, 96));

    DLabel *limkButton = new DLabel(tr("text"));
    limkButton->setText(m_linkName);
    limkButton->setFont(fontType);

    QVBoxLayout *overallLayout = new QVBoxLayout;
    overallLayout->setContentsMargins(0, 0, 0, 0);
    overallLayout->addStretch();
    overallLayout->addWidget(titleIcon, 0, Qt::AlignCenter);
    overallLayout->addSpacing(14);
    overallLayout->addWidget(limkButton, 0, Qt::AlignCenter);
    overallLayout->addStretch();
    this->setLayout(overallLayout);

    this->installEventFilter(this);
    titleIcon->setAttribute(Qt::WA_TransparentForMouseEvents);
}

SelfSupportModule::~SelfSupportModule()
{
}

void SelfSupportModule::setUrl(QString url)
{
    m_linkAddress = url;
}

bool SelfSupportModule::eventFilter(QObject *object, QEvent *event)
{
    if (object == this) {
        if (event->type() == QEvent::MouseButtonRelease) {
            QDesktopServices::openUrl(QUrl(m_linkAddress));
        }
    }

    return QFrame::eventFilter(object, event);
}
