/*
 * Copyright (C) 2019 ~ 2020 Uniontech Software Technology Co.,Ltd.
 *
 * Author:     zhangminjie <zhangminjie@uniontech.com>
 *
 * Maintainer: zhangminjie <zhangminjie@uniontech.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "selfsupport.h"

#include <DCommandLinkButton>
#include <DLabel>

#include <QHBoxLayout>
#include <QDesktopServices>
#include <QUrl>

SelfSupport::SelfSupport(QWidget *parent)
    : DWidget(parent)
    , m_linkUrl("https://www.uniontech.com/service")
{
    QFont fontType;
    fontType.setPointSize(16);
    fontType.setBold(true);

    DLabel *supportLabel = new DLabel(tr("Self-Service"), this);
    supportLabel->setAlignment(Qt::AlignCenter);
    supportLabel->setFont(fontType);

    QString banner;

    if (QLocale::system().name() == "zh_CN") {
        banner = "uss_banner";
    } else {
        banner = "uss_bannerEnglish";
    }

    QFont titleLabelFont;
    titleLabelFont.setPointSize(9);
    DPalette titleLabelColor;
    titleLabelColor.setColor(DPalette::TextTips, QColor("#8AA1B4"));

    DLabel *titleLabel = new DLabel;
    titleLabel->setText(tr("Search and read the manual, document center and FAQ to resolve issues."));
    titleLabel->setFont(titleLabelFont);
    titleLabel->setPalette(titleLabelColor);

    DLabel *comprehendLabel = new DLabel;
    comprehendLabel->setText(tr("Learn about the"));
    comprehendLabel->setFont(titleLabelFont);
    comprehendLabel->setPalette(titleLabelColor);

    DCommandLinkButton *linkButton = new DCommandLinkButton(tr(""));
    linkButton->setText(tr("Privacy Policy"));
    linkButton->setFont(titleLabelFont);
    linkButton->setFocusPolicy(Qt::NoFocus);

    DLabel *andLabel = new DLabel;
    andLabel->setText(tr("and"));
    andLabel->setFont(titleLabelFont);
    andLabel->setPalette(titleLabelColor);

    DCommandLinkButton *idButton = new DCommandLinkButton(tr(""));
    idButton->setText(tr("Union ID Service Agreement"));
    idButton->setFont(titleLabelFont);
    idButton->setFocusPolicy(Qt::NoFocus);

    connect(linkButton, &DCommandLinkButton::clicked, this, [ = ]() {
        QDesktopServices::openUrl(QUrl("https://www.uniontech.com/agreement/privacy-cn"));
    });

    connect(idButton, &DCommandLinkButton::clicked, this, [ = ]() {
        QDesktopServices::openUrl(QUrl("https://www.uniontech.com/agreement/agreement-cn"));
    });

    QString documentCenterIcon = "uss_documentCenter";
    QString documentCenterTitle = tr("Document Center");
    QString documentCenterLink = "https://doc.chinauos.com";

    QString goFAQIcon = "uss_FAQ";
    QString goFAQTitle = tr("FAQ");
    QString goFAQLink = "https://doc.chinauos.com";

    QString SupportIcon = "uss_people";
    QString SupportTitle = tr("Read More");
    QString SupportLink = m_linkUrl;

    SelfSupportModule *documentCenter = new SelfSupportModule(this, documentCenterIcon, documentCenterTitle, documentCenterLink);
    documentCenter->setBackgroundRole(DPalette::ItemBackground);

    SelfSupportModule *goFAQ = new SelfSupportModule(this, goFAQIcon, goFAQTitle, goFAQLink);
    goFAQ->setBackgroundRole(DPalette::ItemBackground);

    openHelpManual *helpManual = new openHelpManual;
    helpManual->setBackgroundRole(DPalette::ItemBackground);

    m_readMore = new SelfSupportModule(this, SupportIcon, SupportTitle, SupportLink);
    m_readMore->setBackgroundRole(DPalette::ItemBackground);

    QHBoxLayout *horizontalLayout = new QHBoxLayout;
    horizontalLayout->setContentsMargins(0, 0, 0, 0);
    horizontalLayout->addStretch();
    horizontalLayout->addSpacing(10);
    horizontalLayout->addWidget(documentCenter);
    horizontalLayout->addSpacing(10);
    horizontalLayout->addWidget(goFAQ);
    horizontalLayout->addSpacing(10);
    horizontalLayout->addWidget(helpManual);
    horizontalLayout->addSpacing(10);
    horizontalLayout->addWidget(m_readMore);
    horizontalLayout->addSpacing(10);
    horizontalLayout->addStretch();

    QVBoxLayout *verticalLayout = new QVBoxLayout;
    verticalLayout->setContentsMargins(0, 0, 0, 0);
    verticalLayout->addSpacing(60);
    verticalLayout->addWidget(supportLabel);
    verticalLayout->addSpacing(6);
    verticalLayout->addWidget(titleLabel, 0, Qt::AlignCenter);

    QVBoxLayout *overallLayout = new QVBoxLayout;
    QHBoxLayout *bottomLayout = new QHBoxLayout;

    overallLayout->setContentsMargins(0, 0, 0, 0);
    overallLayout->addLayout(verticalLayout);
    overallLayout->addSpacing(44);
    overallLayout->addLayout(horizontalLayout);
    overallLayout->addStretch();
    overallLayout->addLayout(bottomLayout);
    bottomLayout->addStretch();
    bottomLayout->addWidget(comprehendLabel);
    bottomLayout->addWidget(linkButton);
    bottomLayout->addWidget(andLabel);
    bottomLayout->addWidget(idButton);
    bottomLayout->addStretch();
    overallLayout->addSpacing(30);
    setLayout(overallLayout);
    setLayout(bottomLayout);
}

SelfSupport::~SelfSupport()
{
}

void SelfSupport::updateWebUrl(const QString &linkUrl)
{
    //服务器更新异常，本地加载需求预设地址
    m_linkUrl = "https://www.uniontech.com/service";
    if (!linkUrl.isEmpty() && linkUrl.startsWith("https"))
        m_linkUrl = linkUrl;
    m_readMore->setUrl(m_linkUrl);
}
