/*
 * Copyright (C) 2019 ~ 2020 Uniontech Software Technology Co.,Ltd.
 *
 * Author:     like <like@uniontech.com>
 *
 * Maintainer: like <like@uniontech.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "success.h"

#include <DLabel>
#include <DIconButton>

#include <QVBoxLayout>

SuccessWidget::SuccessWidget(QWidget *parent)
    : DWidget(parent)
{
    initUI();
    connect(m_completeButton, &DSuggestButton::clicked, this, [=]() {
        emit jumpSupportPage();
    });
    connect(m_feedBackAgain, &DPushButton::clicked, this, [=]() {
        emit jumpConsultationPage();
    });
}

SuccessWidget::~SuccessWidget()
{
}

void SuccessWidget::initUI()
{
    this->setFixedSize(620, 580);

    QFont fontType;
    fontType.setPointSize(9);

    DLabel *successImage = new DLabel;
    successImage->setPixmap(QIcon::fromTheme("uss_success").pixmap(QSize(140, 140)));

    DLabel *successLabel = new DLabel;
    QFont successFont("SourceHanSansSC-Bold,SourceHanSansSC", 12, 75);
    successLabel->setText(tr("Submission Successful"));
    successLabel->setFont(successFont);

    DLabel *contentLabel = new DLabel;
    contentLabel->setText(tr("Please pay attention to your email, we will send the progress and results there."));
    contentLabel->setFont(fontType);
    contentLabel->setAlignment(Qt::AlignHCenter);

    m_feedBackAgain = new DPushButton;
    m_feedBackAgain->setFixedSize(170, 36);
    m_feedBackAgain->setFocusPolicy(Qt::NoFocus);
    m_feedBackAgain->setText(tr("Write Again"));
    m_feedBackAgain->setFont(fontType);

    m_completeButton = new DSuggestButton;
    m_completeButton->setFixedSize(170, 36);
    m_completeButton->setFocusPolicy(Qt::NoFocus);
    m_completeButton->setText(tr("Done"));
    m_completeButton->setFont(fontType);

    QHBoxLayout *buttonLayout = new QHBoxLayout;
    buttonLayout->addStretch();
    buttonLayout->addWidget(m_feedBackAgain);
    buttonLayout->addSpacing(14);
    buttonLayout->addWidget(m_completeButton);
    buttonLayout->addStretch();

    QVBoxLayout *overallLayout = new QVBoxLayout;
    overallLayout->addSpacing(85);
    overallLayout->addWidget(successImage, 0, Qt::AlignCenter);
    overallLayout->addSpacing(5);
    overallLayout->addWidget(successLabel, 0, Qt::AlignCenter);
    overallLayout->addSpacing(2);
    overallLayout->addWidget(contentLabel, 0, Qt::AlignCenter);
    overallLayout->addStretch();
    overallLayout->addLayout(buttonLayout);
    overallLayout->addSpacing(1);
    this->setLayout(overallLayout);
}
