/*
 * Copyright (C) 2019 ~ 2020 Uniontech Software Technology Co.,Ltd.
 *
 * Author:     zhangminjie <zhangminjie@uniontech.com>
 *
 * Maintainer: zhangminjie <zhangminjie@uniontech.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "userlogin.h"
#include "common/deepinaccount.h"

#include <DLabel>
#include <DIconButton>
#include <DCommandLinkButton>

#include <QDesktopServices>
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QUrl>

UserLoginWidget::UserLoginWidget(QWidget *parent)
    : DWidget(parent)
    , m_user (new DeepinAccount(this))
{
    initUI();
    connect(m_userLogin, &DPushButton::clicked, this, [ = ] () {
            m_user->signUp();
    });
}

UserLoginWidget::~UserLoginWidget()
{

}

void UserLoginWidget::initUI()
{    this->setFixedSize(620, 580);

     QFont fontType;
     fontType.setPointSize(9);

     DLabel *uniontechImage = new DLabel(this);
     uniontechImage->setPixmap(QIcon::fromTheme("uss_uniontech").pixmap(QSize(110, 90)));

     DLabel *signinLabel = new DLabel(this);
     QFont successFont("SourceHanSansSC-Bold,SourceHanSansSC", 12, 75);
     signinLabel->setText(tr("Union ID Sign In"));
     signinLabel->setFont(successFont);

     DLabel *contentLabel = new DLabel(this);
     contentLabel->setFixedSize(324,17);
     contentLabel->setText(tr("To get better services, please sign in"));
     contentLabel->setFont(fontType);
     contentLabel->setAlignment(Qt::AlignHCenter);

     m_userLogin = new DPushButton(this);
     m_userLogin->setFixedSize(302, 36);
     m_userLogin->setFocusPolicy(Qt::NoFocus);
     m_userLogin->setText(tr("Sign In"));
     m_userLogin->setFont(fontType);

     m_register = new DCommandLinkButton(tr(""), this);
     m_register->setText(tr("Register here"));
     m_register->setFocusPolicy(Qt::NoFocus);
     m_register->setFont(fontType);

     connect(m_register, &DCommandLinkButton::clicked, this, [ = ]() {
          QDesktopServices::openUrl(QUrl("https://account.chinauos.com/register?lang=zh"));
      });

     QVBoxLayout *buttonLayout = new QVBoxLayout();
     buttonLayout->addStretch();
     buttonLayout->addWidget(m_userLogin, 0, Qt::AlignCenter);
     buttonLayout->addSpacing(5);
     buttonLayout->addWidget(m_register, 0, Qt::AlignHCenter);
     buttonLayout->addStretch();

     QVBoxLayout *overallLayout = new QVBoxLayout(this);
     overallLayout->addSpacing(148);
     overallLayout->addWidget(uniontechImage, 0, Qt::AlignCenter);
     overallLayout->addSpacing(5);
     overallLayout->addWidget(signinLabel, 0, Qt::AlignCenter);
     overallLayout->addSpacing(2);
     overallLayout->addWidget(contentLabel, 0, Qt::AlignCenter);
     overallLayout->addSpacing(20);
     overallLayout->addLayout(buttonLayout);
     overallLayout->addStretch(30);
     this->setLayout(overallLayout);
}
