/*
 * Copyright (C) 2019 ~ 2020 Uniontech Software Technology Co.,Ltd.
 *
 * Author:     liutianming <liutianming@uniontech.com>
 *
 * Maintainer: liutianming <liutianming@uniontech.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "usertitlebar.h"

#include "common/clientsocket.h"
#include "common/deepinaccount.h"

UserTitleBar::UserTitleBar(QWidget *parent)
    : DIconButton (parent)
    , m_webClient (new DSocketClient("", "", this))
    , m_account (new DeepinAccount(this))
{
    setFixedSize(24, 24);
    setAutoFillBackground(true);
    setContextMenuPolicy(Qt::CustomContextMenu);
    setIcon(QIcon::fromTheme("uss_avatar"));
    setFlat(true);
    setIconSize(QSize(24, 24));

    m_statusmenu = new DMenu(this);
    m_statusmenu->setFixedWidth(130);

    m_userName = new QAction(m_statusmenu);
    m_exitLoad = new QAction(m_statusmenu);

    m_userName->setText("");
    m_userName->setEnabled(false);
    m_exitLoad->setText(tr("Exit"));

    m_statusmenu->addAction(m_userName);
    m_statusmenu->addSeparator();
    m_statusmenu->addAction(m_exitLoad);

    connect(m_exitLoad, &QAction::triggered, this, [ = ] () {
        m_account->exitLogin();
    });

    connect(m_account, &DeepinAccount::loggedIn, this, [ = ] (QString userName, QString avatarPath) {
        m_webClient->getUserAvatar(avatarPath);
        m_userName->setText(userName);
        m_accountName = userName;
        m_avatarPath = avatarPath;
    });

    connect(m_account, &DeepinAccount::loggedOut, this, [ = ] () {
        setIcon(QIcon::fromTheme("uss_avatar"));
        m_account->exitLogin();
    });

    connect(m_webClient, &DSocketClient::avatarPixmap, this, [ = ] (QPixmap pix) {
        setIcon(pix);
    });

    connect(DNetState::getInstance(), &DNetState::noticeNetWork, this, [ = ] () {
        if (DNetState::getInstance()->getNetState() && m_account->loginStatus)
            m_webClient->getUserAvatar(m_avatarPath);

        status = DNetState::getInstance()->getNetState();
    });

    connect(this, &DIconButton::clicked, this, [ = ] () {
        if (!status)
            return;

        if (m_account->loginStatus) {
            auto x = this->rect().left();
            auto y = this->rect().bottom() + 10;
            m_statusmenu->popup(this->mapToGlobal(QPoint(x, y)));
        } else {
            m_account->signUp();
        }
    });

    m_account->initStatus();
    status = DNetState::getInstance()->getNetState();

}

UserTitleBar::~UserTitleBar()
{

}
